@@ -26,25 +26,26 @@ WINE_DEFAULT_DEBUG_CHANNEL(d3d11);
 
 static HRESULT shdr_handler(const char *data, DWORD data_size, DWORD tag, void *ctx)
 {
-    struct d3d_shader_info *shader_info = ctx;
+    struct wined3d_shader_desc *desc = ctx;
     HRESULT hr;
 
     switch (tag)
     {
         case TAG_ISGN:
-            if (FAILED(hr = shader_parse_signature(data, data_size, shader_info->input_signature)))
+            if (FAILED(hr = shader_parse_signature(data, data_size, &desc->input_signature)))
                 return hr;
             break;
 
         case TAG_OSGN:
-            if (FAILED(hr = shader_parse_signature(data, data_size, shader_info->output_signature)))
+            if (FAILED(hr = shader_parse_signature(data, data_size, &desc->output_signature)))
                 return hr;
             break;
 
         case TAG_SHDR:
         case TAG_SHEX:
-            if (shader_info->shader_code) FIXME("Multiple SHDR/SHEX chunks.\n");
-            shader_info->shader_code = (const DWORD *)data;
+            if (desc->byte_code)
+                FIXME("Multiple SHDR/SHEX chunks.\n");
+            desc->byte_code = (const DWORD *)data;
             break;
 
         default:
@@ -55,27 +56,47 @@ static HRESULT shdr_handler(const char *data, DWORD data_size, DWORD tag, void *
     return S_OK;
 }
 
-static HRESULT shader_extract_from_dxbc(const void *dxbc, SIZE_T dxbc_length, struct d3d_shader_info *shader_info)
+static HRESULT shader_extract_from_dxbc(const void *dxbc, SIZE_T dxbc_length, struct wined3d_shader_desc *desc)
 {
     HRESULT hr;
 
-    shader_info->shader_code = NULL;
-    memset(shader_info->input_signature, 0, sizeof(*shader_info->input_signature));
-    memset(shader_info->output_signature, 0, sizeof(*shader_info->output_signature));
+    desc->byte_code = NULL;
+    memset(&desc->input_signature, 0, sizeof(desc->input_signature));
+    memset(&desc->output_signature, 0, sizeof(desc->output_signature));
 
-    hr = parse_dxbc(dxbc, dxbc_length, shdr_handler, shader_info);
-    if (!shader_info->shader_code) hr = E_INVALIDARG;
+    hr = parse_dxbc(dxbc, dxbc_length, shdr_handler, desc);
+    if (!desc->byte_code)
+        hr = E_INVALIDARG;
 
     if (FAILED(hr))
     {
         FIXME("Failed to parse shader, hr %#x.\n", hr);
-        shader_free_signature(shader_info->input_signature);
-        shader_free_signature(shader_info->output_signature);
+        shader_free_signature(&desc->input_signature);
+        shader_free_signature(&desc->output_signature);
     }
 
     return hr;
 }
 
+static const char *shader_get_string(const char *data, size_t data_size, DWORD offset)
+{
+    size_t len, max_len;
+
+    if (offset >= data_size)
+    {
+        WARN("Invalid offset %#x (data size %#lx).\n", offset, (long)data_size);
+        return NULL;
+    }
+
+    max_len = data_size - offset;
+    len = strnlen(data + offset, max_len);
+
+    if (len == max_len)
+        return NULL;
+
+    return data + offset;
+}
+
 HRESULT shader_parse_signature(const char *data, DWORD data_size, struct wined3d_shader_signature *s)
 {
     struct wined3d_shader_signature_element *e;
@@ -83,13 +104,24 @@ HRESULT shader_parse_signature(const char *data, DWORD data_size, struct wined3d
     unsigned int i;
     DWORD count;
 
+    if (!require_space(0, 2, sizeof(DWORD), data_size))
+    {
+        WARN("Invalid data size %#x.\n", data_size);
+        return E_INVALIDARG;
+    }
+
     read_dword(&ptr, &count);
     TRACE("%u elements\n", count);
 
     skip_dword_unknown(&ptr, 1);
 
-    e = HeapAlloc(GetProcessHeap(), 0, count * sizeof(*e));
-    if (!e)
+    if (!require_space(ptr - data, count, 6 * sizeof(DWORD), data_size))
+    {
+        WARN("Invalid count %#x (data size %#x).\n", count, data_size);
+        return E_INVALIDARG;
+    }
+
+    if (!(e = d3d11_calloc(count, sizeof(*e))))
     {
         ERR("Failed to allocate input signature memory.\n");
         return E_OUTOFMEMORY;
@@ -100,7 +132,12 @@ HRESULT shader_parse_signature(const char *data, DWORD data_size, struct wined3d
         UINT name_offset;
 
         read_dword(&ptr, &name_offset);
-        e[i].semantic_name = data + name_offset;
+        if (!(e[i].semantic_name = shader_get_string(data, data_size, name_offset)))
+        {
+            WARN("Invalid name offset %#x (data size %#x).\n", name_offset, data_size);
+            HeapFree(GetProcessHeap(), 0, e);
+            return E_INVALIDARG;
+        }
         read_dword(&ptr, &e[i].semantic_idx);
         read_dword(&ptr, &e[i].sysval_semantic);
         read_dword(&ptr, &e[i].component_type);
@@ -385,9 +422,6 @@ static unsigned int d3d_sm_from_feature_level(D3D_FEATURE_LEVEL feature_level)
 static HRESULT d3d_vertex_shader_init(struct d3d_vertex_shader *shader, struct d3d_device *device,
         const void *byte_code, SIZE_T byte_code_length)
 {
-    struct wined3d_shader_signature output_signature;
-    struct wined3d_shader_signature input_signature;
-    struct d3d_shader_info shader_info;
     struct wined3d_shader_desc desc;
     HRESULT hr;
 
@@ -397,25 +431,19 @@ static HRESULT d3d_vertex_shader_init(struct d3d_vertex_shader *shader, struct d
     wined3d_mutex_lock();
     wined3d_private_store_init(&shader->private_store);
 
-    shader_info.input_signature = &input_signature;
-    shader_info.output_signature = &output_signature;
-    if (FAILED(hr = shader_extract_from_dxbc(byte_code, byte_code_length, &shader_info)))
+    if (FAILED(hr = shader_extract_from_dxbc(byte_code, byte_code_length, &desc)))
     {
         WARN("Failed to extract shader, hr %#x.\n", hr);
         wined3d_private_store_cleanup(&shader->private_store);
         wined3d_mutex_unlock();
         return hr;
     }
-
-    desc.byte_code = shader_info.shader_code;
-    desc.input_signature = &input_signature;
-    desc.output_signature = &output_signature;
     desc.max_version = d3d_sm_from_feature_level(device->feature_level);
 
     hr = wined3d_shader_create_vs(device->wined3d_device, &desc, shader,
             &d3d_vertex_shader_wined3d_parent_ops, &shader->wined3d_shader);
-    shader_free_signature(&input_signature);
-    shader_free_signature(&output_signature);
+    shader_free_signature(&desc.input_signature);
+    shader_free_signature(&desc.output_signature);
     if (FAILED(hr))
     {
         WARN("Failed to create wined3d vertex shader, hr %#x.\n", hr);
@@ -471,6 +499,380 @@ struct d3d_vertex_shader *unsafe_impl_from_ID3D10VertexShader(ID3D10VertexShader
     return impl_from_ID3D10VertexShader(iface);
 }
 
+/* ID3D11HullShader methods */
+
+static inline struct d3d11_hull_shader *impl_from_ID3D11HullShader(ID3D11HullShader *iface)
+{
+    return CONTAINING_RECORD(iface, struct d3d11_hull_shader, ID3D11HullShader_iface);
+}
+
+static HRESULT STDMETHODCALLTYPE d3d11_hull_shader_QueryInterface(ID3D11HullShader *iface,
+        REFIID riid, void **object)
+{
+    TRACE("iface %p, riid %s, object %p.\n", iface, debugstr_guid(riid), object);
+
+    if (IsEqualGUID(riid, &IID_ID3D11HullShader)
+            || IsEqualGUID(riid, &IID_ID3D11DeviceChild)
+            || IsEqualGUID(riid, &IID_IUnknown))
+    {
+        ID3D11HullShader_AddRef(iface);
+        *object = iface;
+        return S_OK;
+    }
+
+    WARN("%s not implemented, returning E_NOINTERFACE.\n", debugstr_guid(riid));
+
+    *object = NULL;
+    return E_NOINTERFACE;
+}
+
+static ULONG STDMETHODCALLTYPE d3d11_hull_shader_AddRef(ID3D11HullShader *iface)
+{
+    struct d3d11_hull_shader *shader = impl_from_ID3D11HullShader(iface);
+    ULONG refcount = InterlockedIncrement(&shader->refcount);
+
+    TRACE("%p increasing refcount to %u.\n", shader, refcount);
+
+    return refcount;
+}
+
+static ULONG STDMETHODCALLTYPE d3d11_hull_shader_Release(ID3D11HullShader *iface)
+{
+    struct d3d11_hull_shader *shader = impl_from_ID3D11HullShader(iface);
+    ULONG refcount = InterlockedDecrement(&shader->refcount);
+
+    TRACE("%p decreasing refcount to %u.\n", shader, refcount);
+
+    if (!refcount)
+    {
+        ID3D11Device *device = shader->device;
+
+        wined3d_mutex_lock();
+        wined3d_shader_decref(shader->wined3d_shader);
+        wined3d_mutex_unlock();
+
+        /* Release the device last, it may cause the wined3d device to be
+         * destroyed. */
+        ID3D11Device_Release(device);
+    }
+
+    return refcount;
+}
+
+static void STDMETHODCALLTYPE d3d11_hull_shader_GetDevice(ID3D11HullShader *iface,
+        ID3D11Device **device)
+{
+    struct d3d11_hull_shader *shader = impl_from_ID3D11HullShader(iface);
+
+    TRACE("iface %p, device %p.\n", iface, device);
+
+    *device = shader->device;
+    ID3D11Device_AddRef(*device);
+}
+
+static HRESULT STDMETHODCALLTYPE d3d11_hull_shader_GetPrivateData(ID3D11HullShader *iface,
+        REFGUID guid, UINT *data_size, void *data)
+{
+    struct d3d11_hull_shader *shader = impl_from_ID3D11HullShader(iface);
+
+    TRACE("iface %p, guid %s, data_size %p, data %p.\n", iface, debugstr_guid(guid), data_size, data);
+
+    return d3d_get_private_data(&shader->private_store, guid, data_size, data);
+}
+
+static HRESULT STDMETHODCALLTYPE d3d11_hull_shader_SetPrivateData(ID3D11HullShader *iface,
+        REFGUID guid, UINT data_size, const void *data)
+{
+    struct d3d11_hull_shader *shader = impl_from_ID3D11HullShader(iface);
+
+    TRACE("iface %p, guid %s, data_size %u, data %p.\n", iface, debugstr_guid(guid), data_size, data);
+
+    return d3d_set_private_data(&shader->private_store, guid, data_size, data);
+}
+
+static HRESULT STDMETHODCALLTYPE d3d11_hull_shader_SetPrivateDataInterface(ID3D11HullShader *iface,
+        REFGUID guid, const IUnknown *data)
+{
+    struct d3d11_hull_shader *shader = impl_from_ID3D11HullShader(iface);
+
+    TRACE("iface %p, guid %s, data %p.\n", iface, debugstr_guid(guid), data);
+
+    return d3d_set_private_data_interface(&shader->private_store, guid, data);
+}
+
+static const struct ID3D11HullShaderVtbl d3d11_hull_shader_vtbl =
+{
+    /* IUnknown methods */
+    d3d11_hull_shader_QueryInterface,
+    d3d11_hull_shader_AddRef,
+    d3d11_hull_shader_Release,
+    /* ID3D11DeviceChild methods */
+    d3d11_hull_shader_GetDevice,
+    d3d11_hull_shader_GetPrivateData,
+    d3d11_hull_shader_SetPrivateData,
+    d3d11_hull_shader_SetPrivateDataInterface,
+};
+
+static void STDMETHODCALLTYPE d3d11_hull_shader_wined3d_object_destroyed(void *parent)
+{
+    struct d3d11_hull_shader *shader = parent;
+
+    wined3d_private_store_cleanup(&shader->private_store);
+    HeapFree(GetProcessHeap(), 0, parent);
+}
+
+static const struct wined3d_parent_ops d3d11_hull_shader_wined3d_parent_ops =
+{
+    d3d11_hull_shader_wined3d_object_destroyed,
+};
+
+static HRESULT d3d11_hull_shader_init(struct d3d11_hull_shader *shader, struct d3d_device *device,
+        const void *byte_code, SIZE_T byte_code_length)
+{
+    struct wined3d_shader_desc desc;
+    HRESULT hr;
+
+    shader->ID3D11HullShader_iface.lpVtbl = &d3d11_hull_shader_vtbl;
+    shader->refcount = 1;
+    wined3d_mutex_lock();
+    wined3d_private_store_init(&shader->private_store);
+
+    if (FAILED(hr = shader_extract_from_dxbc(byte_code, byte_code_length, &desc)))
+    {
+        WARN("Failed to extract shader, hr %#x.\n", hr);
+        wined3d_private_store_cleanup(&shader->private_store);
+        wined3d_mutex_unlock();
+        return hr;
+    }
+    desc.max_version = d3d_sm_from_feature_level(device->feature_level);
+
+    hr = wined3d_shader_create_hs(device->wined3d_device, &desc, shader,
+            &d3d11_hull_shader_wined3d_parent_ops, &shader->wined3d_shader);
+    shader_free_signature(&desc.input_signature);
+    shader_free_signature(&desc.output_signature);
+    if (FAILED(hr))
+    {
+        WARN("Failed to create wined3d hull shader, hr %#x.\n", hr);
+        wined3d_private_store_cleanup(&shader->private_store);
+        wined3d_mutex_unlock();
+        return E_INVALIDARG;
+    }
+    wined3d_mutex_unlock();
+
+    shader->device = &device->ID3D11Device_iface;
+    ID3D11Device_AddRef(shader->device);
+
+    return S_OK;
+}
+
+HRESULT d3d11_hull_shader_create(struct d3d_device *device, const void *byte_code, SIZE_T byte_code_length,
+        struct d3d11_hull_shader **shader)
+{
+    struct d3d11_hull_shader *object;
+    HRESULT hr;
+
+    if (!(object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object))))
+        return E_OUTOFMEMORY;
+
+    if (FAILED(hr = d3d11_hull_shader_init(object, device, byte_code, byte_code_length)))
+    {
+        HeapFree(GetProcessHeap(), 0, object);
+        return hr;
+    }
+
+    TRACE("Created hull shader %p.\n", object);
+    *shader = object;
+
+    return S_OK;
+}
+
+/* ID3D11DomainShader methods */
+
+static inline struct d3d11_domain_shader *impl_from_ID3D11DomainShader(ID3D11DomainShader *iface)
+{
+    return CONTAINING_RECORD(iface, struct d3d11_domain_shader, ID3D11DomainShader_iface);
+}
+
+static HRESULT STDMETHODCALLTYPE d3d11_domain_shader_QueryInterface(ID3D11DomainShader *iface,
+        REFIID riid, void **object)
+{
+    TRACE("iface %p, riid %s, object %p.\n", iface, debugstr_guid(riid), object);
+
+    if (IsEqualGUID(riid, &IID_ID3D11DomainShader)
+            || IsEqualGUID(riid, &IID_ID3D11DeviceChild)
+            || IsEqualGUID(riid, &IID_IUnknown))
+    {
+        ID3D11DomainShader_AddRef(iface);
+        *object = iface;
+        return S_OK;
+    }
+
+    WARN("%s not implemented, returning E_NOINTERFACE.\n", debugstr_guid(riid));
+
+    *object = NULL;
+    return E_NOINTERFACE;
+}
+
+static ULONG STDMETHODCALLTYPE d3d11_domain_shader_AddRef(ID3D11DomainShader *iface)
+{
+    struct d3d11_domain_shader *shader = impl_from_ID3D11DomainShader(iface);
+    ULONG refcount = InterlockedIncrement(&shader->refcount);
+
+    TRACE("%p increasing refcount to %u.\n", shader, refcount);
+
+    return refcount;
+}
+
+static ULONG STDMETHODCALLTYPE d3d11_domain_shader_Release(ID3D11DomainShader *iface)
+{
+    struct d3d11_domain_shader *shader = impl_from_ID3D11DomainShader(iface);
+    ULONG refcount = InterlockedDecrement(&shader->refcount);
+
+    TRACE("%p decreasing refcount to %u.\n", shader, refcount);
+
+    if (!refcount)
+    {
+        ID3D11Device *device = shader->device;
+
+        wined3d_mutex_lock();
+        wined3d_shader_decref(shader->wined3d_shader);
+        wined3d_mutex_unlock();
+
+        /* Release the device last, it may cause the wined3d device to be
+         * destroyed. */
+        ID3D11Device_Release(device);
+    }
+
+    return refcount;
+}
+
+static void STDMETHODCALLTYPE d3d11_domain_shader_GetDevice(ID3D11DomainShader *iface,
+        ID3D11Device **device)
+{
+    struct d3d11_domain_shader *shader = impl_from_ID3D11DomainShader(iface);
+
+    TRACE("iface %p, device %p.\n", iface, device);
+
+    *device = shader->device;
+    ID3D11Device_AddRef(*device);
+}
+
+static HRESULT STDMETHODCALLTYPE d3d11_domain_shader_GetPrivateData(ID3D11DomainShader *iface,
+        REFGUID guid, UINT *data_size, void *data)
+{
+    struct d3d11_domain_shader *shader = impl_from_ID3D11DomainShader(iface);
+
+    TRACE("iface %p, guid %s, data_size %p, data %p.\n", iface, debugstr_guid(guid), data_size, data);
+
+    return d3d_get_private_data(&shader->private_store, guid, data_size, data);
+}
+
+static HRESULT STDMETHODCALLTYPE d3d11_domain_shader_SetPrivateData(ID3D11DomainShader *iface,
+        REFGUID guid, UINT data_size, const void *data)
+{
+    struct d3d11_domain_shader *shader = impl_from_ID3D11DomainShader(iface);
+
+    TRACE("iface %p, guid %s, data_size %u, data %p.\n", iface, debugstr_guid(guid), data_size, data);
+
+    return d3d_set_private_data(&shader->private_store, guid, data_size, data);
+}
+
+static HRESULT STDMETHODCALLTYPE d3d11_domain_shader_SetPrivateDataInterface(ID3D11DomainShader *iface,
+        REFGUID guid, const IUnknown *data)
+{
+    struct d3d11_domain_shader *shader = impl_from_ID3D11DomainShader(iface);
+
+    TRACE("iface %p, guid %s, data %p.\n", iface, debugstr_guid(guid), data);
+
+    return d3d_set_private_data_interface(&shader->private_store, guid, data);
+}
+
+static const struct ID3D11DomainShaderVtbl d3d11_domain_shader_vtbl =
+{
+    /* IUnknown methods */
+    d3d11_domain_shader_QueryInterface,
+    d3d11_domain_shader_AddRef,
+    d3d11_domain_shader_Release,
+    /* ID3D11DeviceChild methods */
+    d3d11_domain_shader_GetDevice,
+    d3d11_domain_shader_GetPrivateData,
+    d3d11_domain_shader_SetPrivateData,
+    d3d11_domain_shader_SetPrivateDataInterface,
+};
+
+static void STDMETHODCALLTYPE d3d11_domain_shader_wined3d_object_destroyed(void *parent)
+{
+    struct d3d11_domain_shader *shader = parent;
+
+    wined3d_private_store_cleanup(&shader->private_store);
+    HeapFree(GetProcessHeap(), 0, parent);
+}
+
+static const struct wined3d_parent_ops d3d11_domain_shader_wined3d_parent_ops =
+{
+    d3d11_domain_shader_wined3d_object_destroyed,
+};
+
+static HRESULT d3d11_domain_shader_init(struct d3d11_domain_shader *shader, struct d3d_device *device,
+        const void *byte_code, SIZE_T byte_code_length)
+{
+    struct wined3d_shader_desc desc;
+    HRESULT hr;
+
+    shader->ID3D11DomainShader_iface.lpVtbl = &d3d11_domain_shader_vtbl;
+    shader->refcount = 1;
+    wined3d_mutex_lock();
+    wined3d_private_store_init(&shader->private_store);
+
+    if (FAILED(hr = shader_extract_from_dxbc(byte_code, byte_code_length, &desc)))
+    {
+        WARN("Failed to extract shader, hr %#x.\n", hr);
+        wined3d_private_store_cleanup(&shader->private_store);
+        wined3d_mutex_unlock();
+        return hr;
+    }
+    desc.max_version = d3d_sm_from_feature_level(device->feature_level);
+
+    hr = wined3d_shader_create_ds(device->wined3d_device, &desc, shader,
+            &d3d11_domain_shader_wined3d_parent_ops, &shader->wined3d_shader);
+    shader_free_signature(&desc.input_signature);
+    shader_free_signature(&desc.output_signature);
+    if (FAILED(hr))
+    {
+        WARN("Failed to create wined3d domain shader, hr %#x.\n", hr);
+        wined3d_private_store_cleanup(&shader->private_store);
+        wined3d_mutex_unlock();
+        return E_INVALIDARG;
+    }
+    wined3d_mutex_unlock();
+
+    shader->device = &device->ID3D11Device_iface;
+    ID3D11Device_AddRef(shader->device);
+
+    return S_OK;
+}
+
+HRESULT d3d11_domain_shader_create(struct d3d_device *device, const void *byte_code, SIZE_T byte_code_length,
+        struct d3d11_domain_shader **shader)
+{
+    struct d3d11_domain_shader *object;
+    HRESULT hr;
+
+    if (!(object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object))))
+        return E_OUTOFMEMORY;
+
+    if (FAILED(hr = d3d11_domain_shader_init(object, device, byte_code, byte_code_length)))
+    {
+        HeapFree(GetProcessHeap(), 0, object);
+        return hr;
+    }
+
+    TRACE("Created domain shader %p.\n", object);
+    *shader = object;
+
+    return S_OK;
+}
+
 /* ID3D11GeometryShader methods */
 
 static inline struct d3d_geometry_shader *impl_from_ID3D11GeometryShader(ID3D11GeometryShader *iface)
@@ -704,9 +1106,6 @@ static const struct wined3d_parent_ops d3d_geometry_shader_wined3d_parent_ops =
 static HRESULT d3d_geometry_shader_init(struct d3d_geometry_shader *shader, struct d3d_device *device,
         const void *byte_code, SIZE_T byte_code_length)
 {
-    struct wined3d_shader_signature output_signature;
-    struct wined3d_shader_signature input_signature;
-    struct d3d_shader_info shader_info;
     struct wined3d_shader_desc desc;
     HRESULT hr;
 
@@ -716,25 +1115,19 @@ static HRESULT d3d_geometry_shader_init(struct d3d_geometry_shader *shader, stru
     wined3d_mutex_lock();
     wined3d_private_store_init(&shader->private_store);
 
-    shader_info.input_signature = &input_signature;
-    shader_info.output_signature = &output_signature;
-    if (FAILED(hr = shader_extract_from_dxbc(byte_code, byte_code_length, &shader_info)))
+    if (FAILED(hr = shader_extract_from_dxbc(byte_code, byte_code_length, &desc)))
     {
         WARN("Failed to extract shader, hr %#x.\n", hr);
         wined3d_private_store_cleanup(&shader->private_store);
         wined3d_mutex_unlock();
         return hr;
     }
-
-    desc.byte_code = shader_info.shader_code;
-    desc.input_signature = &input_signature;
-    desc.output_signature = &output_signature;
     desc.max_version = d3d_sm_from_feature_level(device->feature_level);
 
     hr = wined3d_shader_create_gs(device->wined3d_device, &desc, shader,
             &d3d_geometry_shader_wined3d_parent_ops, &shader->wined3d_shader);
-    shader_free_signature(&input_signature);
-    shader_free_signature(&output_signature);
+    shader_free_signature(&desc.input_signature);
+    shader_free_signature(&desc.output_signature);
     if (FAILED(hr))
     {
         WARN("Failed to create wined3d geometry shader, hr %#x.\n", hr);
@@ -1030,9 +1423,6 @@ static const struct wined3d_parent_ops d3d_pixel_shader_wined3d_parent_ops =
 static HRESULT d3d_pixel_shader_init(struct d3d_pixel_shader *shader, struct d3d_device *device,
         const void *byte_code, SIZE_T byte_code_length)
 {
-    struct wined3d_shader_signature output_signature;
-    struct wined3d_shader_signature input_signature;
-    struct d3d_shader_info shader_info;
     struct wined3d_shader_desc desc;
     HRESULT hr;
 
@@ -1042,25 +1432,19 @@ static HRESULT d3d_pixel_shader_init(struct d3d_pixel_shader *shader, struct d3d
     wined3d_mutex_lock();
     wined3d_private_store_init(&shader->private_store);
 
-    shader_info.input_signature = &input_signature;
-    shader_info.output_signature = &output_signature;
-    if (FAILED(hr = shader_extract_from_dxbc(byte_code, byte_code_length, &shader_info)))
+    if (FAILED(hr = shader_extract_from_dxbc(byte_code, byte_code_length, &desc)))
     {
         WARN("Failed to extract shader, hr %#x.\n", hr);
         wined3d_private_store_cleanup(&shader->private_store);
         wined3d_mutex_unlock();
         return hr;
     }
-
-    desc.byte_code = shader_info.shader_code;
-    desc.input_signature = &input_signature;
-    desc.output_signature = &output_signature;
     desc.max_version = d3d_sm_from_feature_level(device->feature_level);
 
     hr = wined3d_shader_create_ps(device->wined3d_device, &desc, shader,
             &d3d_pixel_shader_wined3d_parent_ops, &shader->wined3d_shader);
-    shader_free_signature(&input_signature);
-    shader_free_signature(&output_signature);
+    shader_free_signature(&desc.input_signature);
+    shader_free_signature(&desc.output_signature);
     if (FAILED(hr))
     {
         WARN("Failed to create wined3d pixel shader, hr %#x.\n", hr);
@@ -1115,3 +1499,145 @@ struct d3d_pixel_shader *unsafe_impl_from_ID3D10PixelShader(ID3D10PixelShader *i
 
     return impl_from_ID3D10PixelShader(iface);
 }
+
+/* ID3D11ClassLinkage methods */
+
+static inline struct d3d11_class_linkage *impl_from_ID3D11ClassLinkage(ID3D11ClassLinkage *iface)
+{
+    return CONTAINING_RECORD(iface, struct d3d11_class_linkage, ID3D11ClassLinkage_iface);
+}
+
+static HRESULT STDMETHODCALLTYPE d3d11_class_linkage_QueryInterface(ID3D11ClassLinkage *iface,
+        REFIID riid, void **object)
+{
+    TRACE("iface %p, riid %s, object %p.\n", iface, debugstr_guid(riid), object);
+
+    if (IsEqualGUID(riid, &IID_ID3D11ClassLinkage)
+            || IsEqualGUID(riid, &IID_ID3D11DeviceChild)
+            || IsEqualGUID(riid, &IID_IUnknown))
+    {
+        ID3D11ClassLinkage_AddRef(*object = iface);
+        return S_OK;
+    }
+
+    WARN("%s not implemented, returning E_NOINTERFACE.\n", debugstr_guid(riid));
+
+    *object = NULL;
+    return E_NOINTERFACE;
+}
+
+static ULONG STDMETHODCALLTYPE d3d11_class_linkage_AddRef(ID3D11ClassLinkage *iface)
+{
+    struct d3d11_class_linkage *class_linkage = impl_from_ID3D11ClassLinkage(iface);
+    ULONG refcount = InterlockedIncrement(&class_linkage->refcount);
+
+    TRACE("%p increasing refcount to %u.\n", class_linkage, refcount);
+
+    return refcount;
+}
+
+static ULONG STDMETHODCALLTYPE d3d11_class_linkage_Release(ID3D11ClassLinkage *iface)
+{
+    struct d3d11_class_linkage *class_linkage = impl_from_ID3D11ClassLinkage(iface);
+    ULONG refcount = InterlockedDecrement(&class_linkage->refcount);
+
+    TRACE("%p decreasing refcount to %u.\n", class_linkage, refcount);
+
+    if (!refcount)
+    {
+        wined3d_private_store_cleanup(&class_linkage->private_store);
+        HeapFree(GetProcessHeap(), 0, class_linkage);
+    }
+
+    return refcount;
+}
+
+static void STDMETHODCALLTYPE d3d11_class_linkage_GetDevice(ID3D11ClassLinkage *iface,
+        ID3D11Device **device)
+{
+    FIXME("iface %p, device %p stub!\n", iface, device);
+}
+
+static HRESULT STDMETHODCALLTYPE d3d11_class_linkage_GetPrivateData(ID3D11ClassLinkage *iface,
+        REFGUID guid, UINT *data_size, void *data)
+{
+    struct d3d11_class_linkage *class_linkage = impl_from_ID3D11ClassLinkage(iface);
+
+    TRACE("iface %p, guid %s, data_size %p, data %p.\n", iface, debugstr_guid(guid), data_size, data);
+
+    return d3d_get_private_data(&class_linkage->private_store, guid, data_size, data);
+}
+
+static HRESULT STDMETHODCALLTYPE d3d11_class_linkage_SetPrivateData(ID3D11ClassLinkage *iface,
+        REFGUID guid, UINT data_size, const void *data)
+{
+    struct d3d11_class_linkage *class_linkage = impl_from_ID3D11ClassLinkage(iface);
+
+    TRACE("iface %p, guid %s, data_size %u, data %p.\n", iface, debugstr_guid(guid), data_size, data);
+
+    return d3d_set_private_data(&class_linkage->private_store, guid, data_size, data);
+}
+
+static HRESULT STDMETHODCALLTYPE d3d11_class_linkage_SetPrivateDataInterface(ID3D11ClassLinkage *iface,
+        REFGUID guid, const IUnknown *data)
+{
+    struct d3d11_class_linkage *class_linkage = impl_from_ID3D11ClassLinkage(iface);
+
+    TRACE("iface %p, guid %s, data %p.\n", iface, debugstr_guid(guid), data);
+
+    return d3d_set_private_data_interface(&class_linkage->private_store, guid, data);
+}
+
+static HRESULT STDMETHODCALLTYPE d3d11_class_linkage_GetClassInstance(ID3D11ClassLinkage *iface,
+        const char *instance_name, UINT instance_index, ID3D11ClassInstance **class_instance)
+{
+    FIXME("iface %p, instance_name %s, instance_index %u, class_instance %p stub!\n",
+            iface, debugstr_a(instance_name), instance_index, class_instance);
+
+    return E_NOTIMPL;
+}
+
+static HRESULT STDMETHODCALLTYPE d3d11_class_linkage_CreateClassInstance(ID3D11ClassLinkage *iface,
+        const char *type_name, UINT cb_offset, UINT cb_vector_offset, UINT texture_offset,
+        UINT sampler_offset, ID3D11ClassInstance **class_instance)
+{
+    FIXME("iface %p, type_name %s, cb_offset %u, cb_vector_offset %u, texture_offset %u, "
+            "sampler_offset %u, class_instance %p stub!\n",
+            iface, debugstr_a(type_name), cb_offset, cb_vector_offset, texture_offset,
+            sampler_offset, class_instance);
+
+    return E_NOTIMPL;
+}
+
+static const struct ID3D11ClassLinkageVtbl d3d11_class_linkage_vtbl =
+{
+    /* IUnknown methods */
+    d3d11_class_linkage_QueryInterface,
+    d3d11_class_linkage_AddRef,
+    d3d11_class_linkage_Release,
+    /* ID3D11DeviceChild methods */
+    d3d11_class_linkage_GetDevice,
+    d3d11_class_linkage_GetPrivateData,
+    d3d11_class_linkage_SetPrivateData,
+    d3d11_class_linkage_SetPrivateDataInterface,
+    /* ID3D11ClassLinkage methods */
+    d3d11_class_linkage_GetClassInstance,
+    d3d11_class_linkage_CreateClassInstance,
+};
+
+HRESULT d3d11_class_linkage_create(struct d3d_device *device, struct d3d11_class_linkage **class_linkage)
+{
+    struct d3d11_class_linkage *object;
+
+    if (!(object = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*object))))
+        return E_OUTOFMEMORY;
+
+    object->ID3D11ClassLinkage_iface.lpVtbl = &d3d11_class_linkage_vtbl;
+    object->refcount = 1;
+    wined3d_private_store_init(&object->private_store);
+
+    TRACE("Created class linkage %p.\n", object);
+    *class_linkage = object;
+
+    return S_OK;
+}
