@@ -60,7 +60,7 @@ static HRESULT set_dsdesc_from_resource(D3D11_DEPTH_STENCIL_VIEW_DESC *desc, ID3
                 desc->ViewDimension = D3D11_DSV_DIMENSION_TEXTURE1DARRAY;
                 desc->u.Texture1DArray.MipSlice = 0;
                 desc->u.Texture1DArray.FirstArraySlice = 0;
-                desc->u.Texture1DArray.ArraySize = 1;
+                desc->u.Texture1DArray.ArraySize = texture_desc.ArraySize;
             }
 
             return S_OK;
@@ -100,13 +100,13 @@ static HRESULT set_dsdesc_from_resource(D3D11_DEPTH_STENCIL_VIEW_DESC *desc, ID3
                     desc->ViewDimension = D3D11_DSV_DIMENSION_TEXTURE2DARRAY;
                     desc->u.Texture2DArray.MipSlice = 0;
                     desc->u.Texture2DArray.FirstArraySlice = 0;
-                    desc->u.Texture2DArray.ArraySize = 1;
+                    desc->u.Texture2DArray.ArraySize = texture_desc.ArraySize;
                 }
                 else
                 {
                     desc->ViewDimension = D3D11_DSV_DIMENSION_TEXTURE2DMSARRAY;
                     desc->u.Texture2DMSArray.FirstArraySlice = 0;
-                    desc->u.Texture2DMSArray.ArraySize = 1;
+                    desc->u.Texture2DMSArray.ArraySize = texture_desc.ArraySize;
                 }
             }
 
@@ -121,6 +121,59 @@ static HRESULT set_dsdesc_from_resource(D3D11_DEPTH_STENCIL_VIEW_DESC *desc, ID3
     }
 }
 
+static void normalize_dsv_desc(D3D11_DEPTH_STENCIL_VIEW_DESC *desc, ID3D11Resource *resource)
+{
+    D3D11_RESOURCE_DIMENSION dimension;
+    unsigned int layer_count;
+    DXGI_FORMAT format;
+
+    ID3D11Resource_GetType(resource, &dimension);
+    switch (dimension)
+    {
+        case D3D11_RESOURCE_DIMENSION_TEXTURE2D:
+        {
+            const struct d3d_texture2d *texture;
+
+            if (!(texture = unsafe_impl_from_ID3D11Texture2D((ID3D11Texture2D *)resource)))
+            {
+                ERR("Cannot get implementation from ID3D11Texture2D.\n");
+                return;
+            }
+
+            format = texture->desc.Format;
+            layer_count = texture->desc.ArraySize;
+            break;
+        }
+
+        default:
+            return;
+    }
+
+    if (desc->Format == DXGI_FORMAT_UNKNOWN)
+        desc->Format = format;
+
+    switch (desc->ViewDimension)
+    {
+        case D3D11_DSV_DIMENSION_TEXTURE1DARRAY:
+            if (desc->u.Texture1DArray.ArraySize == -1 && desc->u.Texture1DArray.FirstArraySlice < layer_count)
+                desc->u.Texture1DArray.ArraySize = layer_count - desc->u.Texture1DArray.ArraySize;
+            break;
+
+        case D3D11_DSV_DIMENSION_TEXTURE2DARRAY:
+            if (desc->u.Texture2DArray.ArraySize == -1 && desc->u.Texture2DArray.FirstArraySlice < layer_count)
+                desc->u.Texture2DArray.ArraySize = layer_count - desc->u.Texture2DArray.FirstArraySlice;
+            break;
+
+        case D3D11_DSV_DIMENSION_TEXTURE2DMSARRAY:
+            if (desc->u.Texture2DMSArray.ArraySize == -1 && desc->u.Texture2DMSArray.FirstArraySlice < layer_count)
+                desc->u.Texture2DMSArray.ArraySize = layer_count - desc->u.Texture2DMSArray.FirstArraySlice;
+            break;
+
+        default:
+            break;
+    }
+}
+
 static HRESULT set_rtdesc_from_resource(D3D11_RENDER_TARGET_VIEW_DESC *desc, ID3D11Resource *resource)
 {
     D3D11_RESOURCE_DIMENSION dimension;
@@ -156,7 +209,7 @@ static HRESULT set_rtdesc_from_resource(D3D11_RENDER_TARGET_VIEW_DESC *desc, ID3
                 desc->ViewDimension = D3D11_RTV_DIMENSION_TEXTURE1DARRAY;
                 desc->u.Texture1DArray.MipSlice = 0;
                 desc->u.Texture1DArray.FirstArraySlice = 0;
-                desc->u.Texture1DArray.ArraySize = 1;
+                desc->u.Texture1DArray.ArraySize = texture_desc.ArraySize;
             }
 
             return S_OK;
@@ -197,13 +250,13 @@ static HRESULT set_rtdesc_from_resource(D3D11_RENDER_TARGET_VIEW_DESC *desc, ID3
                     desc->ViewDimension = D3D11_RTV_DIMENSION_TEXTURE2DARRAY;
                     desc->u.Texture2DArray.MipSlice = 0;
                     desc->u.Texture2DArray.FirstArraySlice = 0;
-                    desc->u.Texture2DArray.ArraySize = 1;
+                    desc->u.Texture2DArray.ArraySize = texture_desc.ArraySize;
                 }
                 else
                 {
                     desc->ViewDimension = D3D11_RTV_DIMENSION_TEXTURE2DMSARRAY;
                     desc->u.Texture2DMSArray.FirstArraySlice = 0;
-                    desc->u.Texture2DMSArray.ArraySize = 1;
+                    desc->u.Texture2DMSArray.ArraySize = texture_desc.ArraySize;
                 }
             }
 
@@ -229,7 +282,7 @@ static HRESULT set_rtdesc_from_resource(D3D11_RENDER_TARGET_VIEW_DESC *desc, ID3
             desc->ViewDimension = D3D11_RTV_DIMENSION_TEXTURE3D;
             desc->u.Texture3D.MipSlice = 0;
             desc->u.Texture3D.FirstWSlice = 0;
-            desc->u.Texture3D.WSize = 1;
+            desc->u.Texture3D.WSize = texture_desc.Depth;
 
             return S_OK;
         }
@@ -240,6 +293,80 @@ static HRESULT set_rtdesc_from_resource(D3D11_RENDER_TARGET_VIEW_DESC *desc, ID3
     }
 }
 
+static void normalize_rtv_desc(D3D11_RENDER_TARGET_VIEW_DESC *desc, ID3D11Resource *resource)
+{
+    D3D11_RESOURCE_DIMENSION dimension;
+    unsigned int layer_count;
+    DXGI_FORMAT format;
+
+    ID3D11Resource_GetType(resource, &dimension);
+    switch (dimension)
+    {
+        case D3D11_RESOURCE_DIMENSION_TEXTURE2D:
+        {
+            const struct d3d_texture2d *texture;
+
+            if (!(texture = unsafe_impl_from_ID3D11Texture2D((ID3D11Texture2D *)resource)))
+            {
+                ERR("Cannot get implementation from ID3D11Texture2D.\n");
+                return;
+            }
+
+            format = texture->desc.Format;
+            layer_count = texture->desc.ArraySize;
+            break;
+        }
+
+        case D3D11_RESOURCE_DIMENSION_TEXTURE3D:
+        {
+            const struct d3d_texture3d *texture;
+
+            if (!(texture = unsafe_impl_from_ID3D11Texture3D((ID3D11Texture3D *)resource)))
+            {
+                ERR("Cannot get implementation from ID3D11Texture3D.\n");
+                return;
+            }
+
+            format = texture->desc.Format;
+            layer_count = texture->desc.Depth;
+            break;
+        }
+
+        default:
+            return;
+    }
+
+    if (desc->Format == DXGI_FORMAT_UNKNOWN)
+        desc->Format = format;
+
+    switch (desc->ViewDimension)
+    {
+        case D3D11_RTV_DIMENSION_TEXTURE1DARRAY:
+            if (desc->u.Texture1DArray.ArraySize == -1 && desc->u.Texture1DArray.FirstArraySlice < layer_count)
+                desc->u.Texture1DArray.ArraySize = layer_count - desc->u.Texture1DArray.ArraySize;
+            break;
+
+        case D3D11_RTV_DIMENSION_TEXTURE2DARRAY:
+            if (desc->u.Texture2DArray.ArraySize == -1 && desc->u.Texture2DArray.FirstArraySlice < layer_count)
+                desc->u.Texture2DArray.ArraySize = layer_count - desc->u.Texture2DArray.FirstArraySlice;
+            break;
+
+        case D3D11_RTV_DIMENSION_TEXTURE2DMSARRAY:
+            if (desc->u.Texture2DMSArray.ArraySize == -1 && desc->u.Texture2DMSArray.FirstArraySlice < layer_count)
+                desc->u.Texture2DMSArray.ArraySize = layer_count - desc->u.Texture2DMSArray.FirstArraySlice;
+            break;
+
+        case D3D11_RTV_DIMENSION_TEXTURE3D:
+            layer_count = max(1, layer_count >> desc->u.Texture3D.MipSlice);
+            if (desc->u.Texture3D.WSize == -1 && desc->u.Texture3D.FirstWSlice < layer_count)
+                desc->u.Texture3D.WSize = layer_count - desc->u.Texture3D.FirstWSlice;
+            break;
+
+        default:
+            break;
+    }
+}
+
 static HRESULT set_srdesc_from_resource(D3D11_SHADER_RESOURCE_VIEW_DESC *desc, ID3D11Resource *resource)
 {
     D3D11_RESOURCE_DIMENSION dimension;
@@ -359,6 +486,107 @@ static HRESULT set_srdesc_from_resource(D3D11_SHADER_RESOURCE_VIEW_DESC *desc, I
     }
 }
 
+static void normalize_srv_desc(D3D11_SHADER_RESOURCE_VIEW_DESC *desc, ID3D11Resource *resource)
+{
+    unsigned int miplevel_count, layer_count;
+    D3D11_RESOURCE_DIMENSION dimension;
+    DXGI_FORMAT format;
+
+    ID3D11Resource_GetType(resource, &dimension);
+    switch (dimension)
+    {
+        case D3D11_RESOURCE_DIMENSION_TEXTURE2D:
+        {
+            const struct d3d_texture2d *texture;
+
+            if (!(texture = unsafe_impl_from_ID3D11Texture2D((ID3D11Texture2D *)resource)))
+            {
+                ERR("Cannot get implementation from ID3D11Texture2D.\n");
+                return;
+            }
+
+            format = texture->desc.Format;
+            miplevel_count = texture->desc.MipLevels;
+            layer_count = texture->desc.ArraySize;
+            break;
+        }
+
+        case D3D11_RESOURCE_DIMENSION_TEXTURE3D:
+        {
+            const struct d3d_texture3d *texture;
+
+            if (!(texture = unsafe_impl_from_ID3D11Texture3D((ID3D11Texture3D *)resource)))
+            {
+                ERR("Cannot get implementation from ID3D11Texture3D.\n");
+                return;
+            }
+
+            format = texture->desc.Format;
+            miplevel_count = texture->desc.MipLevels;
+            layer_count = 1;
+            break;
+        }
+
+        default:
+            return;
+    }
+
+    if (desc->Format == DXGI_FORMAT_UNKNOWN)
+        desc->Format = format;
+
+    switch (desc->ViewDimension)
+    {
+        case D3D11_SRV_DIMENSION_TEXTURE1D:
+            if (desc->u.Texture1D.MipLevels == -1 && desc->u.Texture1D.MostDetailedMip < miplevel_count)
+                desc->u.Texture1D.MipLevels = miplevel_count - desc->u.Texture1D.MostDetailedMip;
+            break;
+
+        case D3D11_SRV_DIMENSION_TEXTURE1DARRAY:
+            if (desc->u.Texture1DArray.MipLevels == -1 && desc->u.Texture1DArray.MostDetailedMip < miplevel_count)
+                desc->u.Texture1DArray.MipLevels = miplevel_count - desc->u.Texture1DArray.MostDetailedMip;
+            if (desc->u.Texture1DArray.ArraySize == -1 && desc->u.Texture1DArray.FirstArraySlice < miplevel_count)
+                desc->u.Texture1DArray.ArraySize = layer_count - desc->u.Texture1DArray.FirstArraySlice;
+            break;
+
+        case D3D11_SRV_DIMENSION_TEXTURE2D:
+            if (desc->u.Texture2D.MipLevels == -1 && desc->u.Texture2D.MostDetailedMip < miplevel_count)
+                desc->u.Texture2D.MipLevels = miplevel_count - desc->u.Texture2D.MostDetailedMip;
+            break;
+
+        case D3D11_SRV_DIMENSION_TEXTURE2DARRAY:
+            if (desc->u.Texture2DArray.MipLevels == -1 && desc->u.Texture2DArray.MostDetailedMip < miplevel_count)
+                desc->u.Texture2DArray.MipLevels = miplevel_count - desc->u.Texture2DArray.MostDetailedMip;
+            if (desc->u.Texture2DArray.ArraySize == -1 && desc->u.Texture2DArray.FirstArraySlice < layer_count)
+                desc->u.Texture2DArray.ArraySize = layer_count - desc->u.Texture2DArray.FirstArraySlice;
+            break;
+
+        case D3D11_SRV_DIMENSION_TEXTURE2DMSARRAY:
+            if (desc->u.Texture2DMSArray.ArraySize == -1 && desc->u.Texture2DMSArray.FirstArraySlice < layer_count)
+                desc->u.Texture2DMSArray.ArraySize = layer_count - desc->u.Texture2DMSArray.FirstArraySlice;
+            break;
+
+        case D3D11_SRV_DIMENSION_TEXTURE3D:
+            if (desc->u.Texture3D.MipLevels == -1 && desc->u.Texture3D.MostDetailedMip < miplevel_count)
+                desc->u.Texture3D.MipLevels = miplevel_count - desc->u.Texture3D.MostDetailedMip;
+            break;
+
+        case D3D11_SRV_DIMENSION_TEXTURECUBE:
+            if (desc->u.TextureCube.MipLevels == -1 && desc->u.TextureCube.MostDetailedMip < miplevel_count)
+                desc->u.TextureCube.MipLevels = miplevel_count - desc->u.TextureCube.MostDetailedMip;
+            break;
+
+        case D3D11_SRV_DIMENSION_TEXTURECUBEARRAY:
+            if (desc->u.TextureCubeArray.MipLevels == -1 && desc->u.TextureCubeArray.MostDetailedMip < miplevel_count)
+                desc->u.TextureCubeArray.MipLevels = miplevel_count - desc->u.TextureCubeArray.MostDetailedMip;
+            if (desc->u.TextureCubeArray.NumCubes == -1 && desc->u.TextureCubeArray.First2DArrayFace < layer_count)
+                desc->u.TextureCubeArray.NumCubes = (layer_count - desc->u.TextureCubeArray.First2DArrayFace) / 6;
+            break;
+
+        default:
+            break;
+    }
+}
+
 /* ID3D11DepthStencilView methods */
 
 static inline struct d3d_depthstencil_view *impl_from_ID3D11DepthStencilView(ID3D11DepthStencilView *iface)
@@ -706,6 +934,7 @@ static HRESULT d3d_depthstencil_view_init(struct d3d_depthstencil_view *view, st
     else
     {
         view->desc = *desc;
+        normalize_dsv_desc(&view->desc, resource);
     }
 
     wined3d_mutex_lock();
@@ -1121,12 +1350,13 @@ static HRESULT d3d_rendertarget_view_init(struct d3d_rendertarget_view *view, st
 
     if (!desc)
     {
-        HRESULT hr = set_rtdesc_from_resource(&view->desc, resource);
-        if (FAILED(hr)) return hr;
+        if (FAILED(hr = set_rtdesc_from_resource(&view->desc, resource)))
+            return hr;
     }
     else
     {
         view->desc = *desc;
+        normalize_rtv_desc(&view->desc, resource);
     }
 
     wined3d_mutex_lock();
@@ -1480,9 +1710,105 @@ static const struct ID3D10ShaderResourceView1Vtbl d3d10_shader_resource_view_vtb
     d3d10_shader_resource_view_GetDesc1,
 };
 
+static HRESULT wined3d_shader_resource_view_desc_from_d3d11(struct wined3d_shader_resource_view_desc *wined3d_desc,
+        const D3D11_SHADER_RESOURCE_VIEW_DESC *desc)
+{
+    wined3d_desc->format_id = wined3dformat_from_dxgi_format(desc->Format);
+    wined3d_desc->flags = 0;
+
+    switch (desc->ViewDimension)
+    {
+        case D3D11_SRV_DIMENSION_BUFFER:
+            wined3d_desc->u.buffer.start_idx = desc->u.Buffer.u1.FirstElement;
+            wined3d_desc->u.buffer.count = desc->u.Buffer.u2.NumElements;
+            break;
+
+        case D3D11_SRV_DIMENSION_TEXTURE1D:
+            wined3d_desc->u.texture.level_idx = desc->u.Texture1D.MostDetailedMip;
+            wined3d_desc->u.texture.level_count = desc->u.Texture1D.MipLevels;
+            wined3d_desc->u.texture.layer_idx = 0;
+            wined3d_desc->u.texture.layer_count = 1;
+            break;
+
+        case D3D11_SRV_DIMENSION_TEXTURE1DARRAY:
+            wined3d_desc->flags = WINED3D_VIEW_TEXTURE_ARRAY;
+            wined3d_desc->u.texture.level_idx = desc->u.Texture1DArray.MostDetailedMip;
+            wined3d_desc->u.texture.level_count = desc->u.Texture1DArray.MipLevels;
+            wined3d_desc->u.texture.layer_idx = desc->u.Texture1DArray.FirstArraySlice;
+            wined3d_desc->u.texture.layer_count = desc->u.Texture1DArray.ArraySize;
+            break;
+
+        case D3D11_SRV_DIMENSION_TEXTURE2D:
+            wined3d_desc->u.texture.level_idx = desc->u.Texture2D.MostDetailedMip;
+            wined3d_desc->u.texture.level_count = desc->u.Texture2D.MipLevels;
+            wined3d_desc->u.texture.layer_idx = 0;
+            wined3d_desc->u.texture.layer_count = 1;
+            break;
+
+        case D3D11_SRV_DIMENSION_TEXTURE2DARRAY:
+            wined3d_desc->flags = WINED3D_VIEW_TEXTURE_ARRAY;
+            wined3d_desc->u.texture.level_idx = desc->u.Texture2DArray.MostDetailedMip;
+            wined3d_desc->u.texture.level_count = desc->u.Texture2DArray.MipLevels;
+            wined3d_desc->u.texture.layer_idx = desc->u.Texture2DArray.FirstArraySlice;
+            wined3d_desc->u.texture.layer_count = desc->u.Texture2DArray.ArraySize;
+            break;
+
+        case D3D11_SRV_DIMENSION_TEXTURE2DMS:
+            wined3d_desc->u.texture.level_idx = 0;
+            wined3d_desc->u.texture.level_count = 1;
+            wined3d_desc->u.texture.layer_idx = 0;
+            wined3d_desc->u.texture.layer_count = 1;
+            break;
+
+        case D3D11_SRV_DIMENSION_TEXTURE2DMSARRAY:
+            wined3d_desc->flags = WINED3D_VIEW_TEXTURE_ARRAY;
+            wined3d_desc->u.texture.level_idx = 0;
+            wined3d_desc->u.texture.level_count = 1;
+            wined3d_desc->u.texture.layer_idx = desc->u.Texture2DMSArray.FirstArraySlice;
+            wined3d_desc->u.texture.layer_count = desc->u.Texture2DMSArray.ArraySize;
+            break;
+
+        case D3D11_SRV_DIMENSION_TEXTURE3D:
+            wined3d_desc->u.texture.level_idx = desc->u.Texture3D.MostDetailedMip;
+            wined3d_desc->u.texture.level_count = desc->u.Texture3D.MipLevels;
+            wined3d_desc->u.texture.layer_idx = 0;
+            wined3d_desc->u.texture.layer_count = 1;
+            break;
+
+        case D3D11_SRV_DIMENSION_TEXTURECUBE:
+            wined3d_desc->flags = WINED3D_VIEW_TEXTURE_CUBE;
+            wined3d_desc->u.texture.level_idx = desc->u.TextureCube.MostDetailedMip;
+            wined3d_desc->u.texture.level_count = desc->u.TextureCube.MipLevels;
+            wined3d_desc->u.texture.layer_idx = 0;
+            wined3d_desc->u.texture.layer_count = 6;
+            break;
+
+        case D3D11_SRV_DIMENSION_TEXTURECUBEARRAY:
+            wined3d_desc->flags = WINED3D_VIEW_TEXTURE_CUBE | WINED3D_VIEW_TEXTURE_ARRAY;
+            wined3d_desc->u.texture.level_idx = desc->u.TextureCubeArray.MostDetailedMip;
+            wined3d_desc->u.texture.level_count = desc->u.TextureCubeArray.MipLevels;
+            wined3d_desc->u.texture.layer_idx = desc->u.TextureCubeArray.First2DArrayFace;
+            wined3d_desc->u.texture.layer_count = 6 * desc->u.TextureCubeArray.NumCubes;
+            break;
+
+        case D3D11_SRV_DIMENSION_BUFFEREX:
+            wined3d_desc->flags = desc->u.BufferEx.Flags;
+            wined3d_desc->u.buffer.start_idx = desc->u.BufferEx.FirstElement;
+            wined3d_desc->u.buffer.count = desc->u.BufferEx.NumElements;
+            break;
+
+        default:
+            WARN("Unrecognized view dimension %#x.\n", desc->ViewDimension);
+            return E_FAIL;
+    }
+
+    return S_OK;
+}
+
 static HRESULT d3d_shader_resource_view_init(struct d3d_shader_resource_view *view, struct d3d_device *device,
         ID3D11Resource *resource, const D3D11_SHADER_RESOURCE_VIEW_DESC *desc)
 {
+    struct wined3d_shader_resource_view_desc wined3d_desc;
     struct wined3d_resource *wined3d_resource;
     HRESULT hr;
 
@@ -1498,17 +1824,21 @@ static HRESULT d3d_shader_resource_view_init(struct d3d_shader_resource_view *vi
     else
     {
         view->desc = *desc;
+        normalize_srv_desc(&view->desc, resource);
     }
 
+    if (FAILED(hr = wined3d_shader_resource_view_desc_from_d3d11(&wined3d_desc, &view->desc)))
+        return hr;
+
     wined3d_mutex_lock();
     if (!(wined3d_resource = wined3d_resource_from_d3d11_resource(resource)))
     {
         wined3d_mutex_unlock();
-        ERR("Failed to get wined3d resource for d3d10 resource %p.\n", resource);
+        ERR("Failed to get wined3d resource for d3d11 resource %p.\n", resource);
         return E_FAIL;
     }
 
-    if (FAILED(hr = wined3d_shader_resource_view_create(wined3d_resource,
+    if (FAILED(hr = wined3d_shader_resource_view_create(&wined3d_desc, wined3d_resource,
             view, &d3d_null_wined3d_parent_ops, &view->wined3d_view)))
     {
         wined3d_mutex_unlock();
