@@ -471,7 +471,7 @@ typedef struct
 int __cdecl _Mtx_init(_Mtx_t *mtx, int flags)
 {
     if(flags & ~MTX_MULTI_LOCK)
-        FIXME("unknown flags ignorred: %x\n", flags);
+        FIXME("unknown flags ignored: %x\n", flags);
 
     *mtx = MSVCRT_operator_new(sizeof(**mtx));
     (*mtx)->flags = flags;
@@ -487,6 +487,11 @@ void __cdecl _Mtx_destroy(_Mtx_t *mtx)
     MSVCRT_operator_delete(*mtx);
 }
 
+int __cdecl _Mtx_current_owns(_Mtx_t *mtx)
+{
+    return (*mtx)->thread_id == GetCurrentThreadId();
+}
+
 int __cdecl _Mtx_lock(_Mtx_t *mtx)
 {
     if((*mtx)->thread_id != GetCurrentThreadId()) {
