@@ -646,7 +646,7 @@ void CDECL MSVCRT___set_app_type(int app_type)
  */
 char** CDECL _get_initial_narrow_environment(void)
 {
-  return MSVCRT___argv;
+  return MSVCRT___initenv;
 }
 
 /*********************************************************************
@@ -666,3 +666,29 @@ int CDECL _initialize_narrow_environment(void)
   TRACE("\n");
   return 0;
 }
+
+/*********************************************************************
+ *		_get_initial_wide_environment (UCRTBASE.@)
+ */
+MSVCRT_wchar_t** CDECL _get_initial_wide_environment(void)
+{
+  return MSVCRT___winitenv;
+}
+
+/*********************************************************************
+ *		_configure_wide_argv (UCRTBASE.@)
+ */
+int CDECL _configure_wide_argv(int mode)
+{
+  FIXME("(%d) stub\n", mode);
+  return 0;
+}
+
+/*********************************************************************
+ *		_initialize_wide_environment (UCRTBASE.@)
+ */
+int CDECL _initialize_wide_environment(void)
+{
+  FIXME("stub\n");
+  return 0;
+}
