@@ -20,11 +20,6 @@
  */
 #include "config.h"
 #include "wine/port.h"
-#include "winerror.h"
-#include "wine/debug.h"
-
-#include <string.h>
-
 #include "wined3d_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d);
