@@ -27,69 +27,25 @@
 WINE_DEFAULT_DEBUG_CHANNEL(d3d);
 WINE_DECLARE_DEBUG_CHANNEL(d3d_perf);
 
-static BOOL volume_prepare_system_memory(struct wined3d_volume *volume)
-{
-    if (volume->resource.heap_memory)
-        return TRUE;
-
-    if (!wined3d_resource_allocate_sysmem(&volume->resource))
-    {
-        ERR("Failed to allocate system memory.\n");
-        return FALSE;
-    }
-    return TRUE;
-}
-
-/* Context activation is done by the caller. Context may be NULL in
- * WINED3D_NO3D mode. */
-BOOL wined3d_volume_prepare_location(struct wined3d_volume *volume,
-        struct wined3d_context *context, DWORD location)
-{
-    struct wined3d_texture *texture = volume->container;
-
-    switch (location)
-    {
-        case WINED3D_LOCATION_SYSMEM:
-            return volume_prepare_system_memory(volume);
-
-        case WINED3D_LOCATION_BUFFER:
-            wined3d_texture_prepare_buffer_object(texture, volume->texture_level, context->gl_info);
-            return TRUE;
-
-        case WINED3D_LOCATION_TEXTURE_RGB:
-            wined3d_texture_prepare_texture(texture, context, FALSE);
-            return TRUE;
-
-        case WINED3D_LOCATION_TEXTURE_SRGB:
-            wined3d_texture_prepare_texture(texture, context, TRUE);
-            return TRUE;
-
-        default:
-            ERR("Invalid location %s.\n", wined3d_debug_location(location));
-            return FALSE;
-    }
-}
-
 /* This call just uploads data, the caller is responsible for binding the
  * correct texture. */
 /* Context activation is done by the caller. */
-void wined3d_volume_upload_data(struct wined3d_volume *volume, const struct wined3d_context *context,
-        const struct wined3d_const_bo_address *data)
+void wined3d_volume_upload_data(struct wined3d_texture *texture, unsigned int sub_resource_idx,
+        const struct wined3d_context *context, const struct wined3d_const_bo_address *data)
 {
-    const struct wined3d_gl_info *gl_info = context->gl_info;
-    struct wined3d_texture *texture = volume->container;
     const struct wined3d_format *format = texture->resource.format;
+    unsigned int level = sub_resource_idx % texture->level_count;
+    const struct wined3d_gl_info *gl_info = context->gl_info;
     unsigned int width, height, depth;
     const void *mem = data->addr;
     void *converted_mem = NULL;
 
-    TRACE("volume %p, context %p, level %u, format %s (%#x).\n",
-            volume, context, volume->texture_level, debug_d3dformat(format->id),
-            format->id);
+    TRACE("texture %p, sub_resource_idx %u, context %p, format %s (%#x).\n",
+            texture, level, context, debug_d3dformat(format->id), format->id);
 
-    width = wined3d_texture_get_level_width(texture, volume->texture_level);
-    height = wined3d_texture_get_level_height(texture, volume->texture_level);
-    depth = wined3d_texture_get_level_depth(texture, volume->texture_level);
+    width = wined3d_texture_get_level_width(texture, level);
+    height = wined3d_texture_get_level_height(texture, level);
+    depth = wined3d_texture_get_level_depth(texture, level);
 
     if (format->convert)
     {
@@ -97,16 +53,16 @@ void wined3d_volume_upload_data(struct wined3d_volume *volume, const struct wine
         UINT src_row_pitch, src_slice_pitch;
 
         if (data->buffer_object)
-            ERR("Loading a converted volume from a PBO.\n");
+            ERR("Loading a converted texture from a PBO.\n");
         if (texture->resource.format_flags & WINED3DFMT_FLAG_BLOCKS)
             ERR("Converting a block-based format.\n");
 
         dst_row_pitch = width * format->conv_byte_count;
         dst_slice_pitch = dst_row_pitch * height;
 
-        wined3d_texture_get_pitch(texture, volume->texture_level, &src_row_pitch, &src_slice_pitch);
+        wined3d_texture_get_pitch(texture, level, &src_row_pitch, &src_slice_pitch);
 
-        converted_mem = HeapAlloc(GetProcessHeap(), 0, dst_slice_pitch * depth);
+        converted_mem = wined3d_calloc(depth, dst_slice_pitch);
         format->convert(data->addr, converted_mem, src_row_pitch, src_slice_pitch,
                 dst_row_pitch, dst_slice_pitch, width, height, depth);
         mem = converted_mem;
@@ -118,9 +74,8 @@ void wined3d_volume_upload_data(struct wined3d_volume *volume, const struct wine
         checkGLcall("glBindBuffer");
     }
 
-    GL_EXTCALL(glTexSubImage3D(GL_TEXTURE_3D, volume->texture_level, 0, 0, 0,
-            width, height, depth,
-            format->glFormat, format->glType, mem));
+    GL_EXTCALL(glTexSubImage3D(GL_TEXTURE_3D, level, 0, 0, 0,
+            width, height, depth, format->glFormat, format->glType, mem));
     checkGLcall("glTexSubImage3D");
 
     if (data->buffer_object)
@@ -164,12 +119,6 @@ static void wined3d_volume_download_data(struct wined3d_volume *volume,
 
 }
 
-static void wined3d_volume_evict_sysmem(struct wined3d_volume *volume)
-{
-    wined3d_resource_free_sysmem(&volume->resource);
-    wined3d_texture_invalidate_location(volume->container, volume->texture_level, WINED3D_LOCATION_SYSMEM);
-}
-
 static DWORD volume_access_from_location(DWORD location)
 {
     switch (location)
@@ -195,6 +144,7 @@ static DWORD volume_access_from_location(DWORD location)
 static void wined3d_volume_srgb_transfer(struct wined3d_volume *volume,
         struct wined3d_context *context, BOOL dest_is_srgb)
 {
+    struct wined3d_texture *texture = volume->container;
     struct wined3d_bo_address data;
     /* Optimizations are possible, but the effort should be put into either
      * implementing EXT_SRGB_DECODE in the driver or finding out why we
@@ -205,32 +155,17 @@ static void wined3d_volume_srgb_transfer(struct wined3d_volume *volume,
 
     WARN_(d3d_perf)("Performing slow rgb/srgb volume transfer.\n");
     data.buffer_object = 0;
-    data.addr = HeapAlloc(GetProcessHeap(), 0, volume->resource.size);
-    if (!data.addr)
+    if (!(data.addr = HeapAlloc(GetProcessHeap(), 0, texture->sub_resources[volume->texture_level].size)))
         return;
 
-    wined3d_texture_bind_and_dirtify(volume->container, context, !dest_is_srgb);
+    wined3d_texture_bind_and_dirtify(texture, context, !dest_is_srgb);
     wined3d_volume_download_data(volume, context, &data);
-    wined3d_texture_bind_and_dirtify(volume->container, context, dest_is_srgb);
-    wined3d_volume_upload_data(volume, context, wined3d_const_bo_address(&data));
+    wined3d_texture_bind_and_dirtify(texture, context, dest_is_srgb);
+    wined3d_volume_upload_data(texture, volume->texture_level, context, wined3d_const_bo_address(&data));
 
     HeapFree(GetProcessHeap(), 0, data.addr);
 }
 
-static BOOL wined3d_volume_can_evict(const struct wined3d_volume *volume)
-{
-    struct wined3d_texture *texture = volume->container;
-
-    if (texture->resource.pool != WINED3D_POOL_MANAGED)
-        return FALSE;
-    if (texture->download_count >= 10)
-        return FALSE;
-    if (texture->resource.format->convert)
-        return FALSE;
-
-    return TRUE;
-}
-
 /* Context activation is done by the caller. */
 BOOL wined3d_volume_load_location(struct wined3d_volume *volume,
         struct wined3d_context *context, DWORD location)
@@ -257,7 +192,7 @@ BOOL wined3d_volume_load_location(struct wined3d_volume *volume,
         return FALSE;
     }
 
-    if (!wined3d_volume_prepare_location(volume, context, location))
+    if (!wined3d_texture_prepare_location(texture, sub_resource_idx, context, location))
         return FALSE;
 
     if (sub_resource->locations & WINED3D_LOCATION_DISCARDED)
@@ -274,17 +209,18 @@ BOOL wined3d_volume_load_location(struct wined3d_volume *volume,
         case WINED3D_LOCATION_TEXTURE_SRGB:
             if (sub_resource->locations & WINED3D_LOCATION_SYSMEM)
             {
-                struct wined3d_const_bo_address data = {0, volume->resource.heap_memory};
+                struct wined3d_const_bo_address data = {0, texture->resource.heap_memory};
+                data.addr += sub_resource->offset;
                 wined3d_texture_bind_and_dirtify(texture, context,
                         location == WINED3D_LOCATION_TEXTURE_SRGB);
-                wined3d_volume_upload_data(volume, context, &data);
+                wined3d_volume_upload_data(texture, sub_resource_idx, context, &data);
             }
             else if (sub_resource->locations & WINED3D_LOCATION_BUFFER)
             {
                 struct wined3d_const_bo_address data = {sub_resource->buffer_object, NULL};
                 wined3d_texture_bind_and_dirtify(texture, context,
                         location == WINED3D_LOCATION_TEXTURE_SRGB);
-                wined3d_volume_upload_data(volume, context, &data);
+                wined3d_volume_upload_data(texture, sub_resource_idx, context, &data);
             }
             else if (sub_resource->locations & WINED3D_LOCATION_TEXTURE_RGB)
             {
@@ -304,8 +240,9 @@ BOOL wined3d_volume_load_location(struct wined3d_volume *volume,
         case WINED3D_LOCATION_SYSMEM:
             if (sub_resource->locations & (WINED3D_LOCATION_TEXTURE_RGB | WINED3D_LOCATION_TEXTURE_SRGB))
             {
-                struct wined3d_bo_address data = {0, volume->resource.heap_memory};
+                struct wined3d_bo_address data = {0, texture->resource.heap_memory};
 
+                data.addr += sub_resource->offset;
                 if (sub_resource->locations & WINED3D_LOCATION_TEXTURE_RGB)
                     wined3d_texture_bind_and_dirtify(texture, context, FALSE);
                 else
@@ -351,116 +288,5 @@ BOOL wined3d_volume_load_location(struct wined3d_volume *volume,
 done:
     wined3d_texture_validate_location(texture, sub_resource_idx, location);
 
-    if (location != WINED3D_LOCATION_SYSMEM && wined3d_volume_can_evict(volume))
-        wined3d_volume_evict_sysmem(volume);
-
     return TRUE;
 }
-
-void wined3d_volume_cleanup(struct wined3d_volume *volume)
-{
-    TRACE("volume %p.\n", volume);
-
-    resource_cleanup(&volume->resource);
-}
-
-static void volume_unload(struct wined3d_resource *resource)
-{
-    struct wined3d_volume *volume = volume_from_resource(resource);
-    struct wined3d_texture *texture = volume->container;
-    struct wined3d_device *device = texture->resource.device;
-    struct wined3d_context *context;
-
-    if (texture->resource.pool == WINED3D_POOL_DEFAULT)
-        ERR("Unloading DEFAULT pool volume.\n");
-
-    TRACE("texture %p.\n", resource);
-
-    context = context_acquire(device, NULL);
-    if (wined3d_volume_load_location(volume, context, WINED3D_LOCATION_SYSMEM))
-    {
-        wined3d_texture_invalidate_location(texture, volume->texture_level, ~WINED3D_LOCATION_SYSMEM);
-    }
-    else
-    {
-        ERR("Out of memory when unloading volume %p.\n", volume);
-        wined3d_texture_validate_location(texture, volume->texture_level, WINED3D_LOCATION_DISCARDED);
-        wined3d_texture_invalidate_location(texture, volume->texture_level, ~WINED3D_LOCATION_DISCARDED);
-    }
-    context_release(context);
-
-    /* The texture name is managed by the container. */
-
-    resource_unload(resource);
-}
-
-static ULONG volume_resource_incref(struct wined3d_resource *resource)
-{
-    struct wined3d_volume *volume = volume_from_resource(resource);
-    TRACE("Forwarding to container %p.\n", volume->container);
-
-    return wined3d_texture_incref(volume->container);
-}
-
-static ULONG volume_resource_decref(struct wined3d_resource *resource)
-{
-    struct wined3d_volume *volume = volume_from_resource(resource);
-    TRACE("Forwarding to container %p.\n", volume->container);
-
-    return wined3d_texture_decref(volume->container);
-}
-
-static HRESULT volume_resource_sub_resource_map(struct wined3d_resource *resource, unsigned int sub_resource_idx,
-        struct wined3d_map_desc *map_desc, const struct wined3d_box *box, DWORD flags)
-{
-    ERR("Not supported on sub-resources.\n");
-    return WINED3DERR_INVALIDCALL;
-}
-
-static HRESULT volume_resource_sub_resource_unmap(struct wined3d_resource *resource, unsigned int sub_resource_idx)
-{
-    ERR("Not supported on sub-resources.\n");
-    return WINED3DERR_INVALIDCALL;
-}
-
-static const struct wined3d_resource_ops volume_resource_ops =
-{
-    volume_resource_incref,
-    volume_resource_decref,
-    volume_unload,
-    volume_resource_sub_resource_map,
-    volume_resource_sub_resource_unmap,
-};
-
-HRESULT wined3d_volume_init(struct wined3d_volume *volume, struct wined3d_texture *container,
-        const struct wined3d_resource_desc *desc, UINT level)
-{
-    struct wined3d_device *device = container->resource.device;
-    const struct wined3d_gl_info *gl_info = &device->adapter->gl_info;
-    const struct wined3d_format *format = wined3d_get_format(gl_info, desc->format);
-    HRESULT hr;
-    UINT size;
-
-    size = wined3d_format_calculate_size(format, device->surface_alignment, desc->width, desc->height, desc->depth);
-
-    if (FAILED(hr = resource_init(&volume->resource, device, WINED3D_RTYPE_VOLUME, format,
-            WINED3D_MULTISAMPLE_NONE, 0, desc->usage, desc->pool, desc->width, desc->height, desc->depth,
-            size, NULL, &wined3d_null_parent_ops, &volume_resource_ops)))
-    {
-        WARN("Failed to initialize resource, returning %#x.\n", hr);
-        return hr;
-    }
-
-    volume->texture_level = level;
-    container->sub_resources[level].locations = WINED3D_LOCATION_DISCARDED;
-
-    if (wined3d_texture_use_pbo(container, gl_info))
-    {
-        wined3d_resource_free_sysmem(&volume->resource);
-        volume->resource.map_binding = WINED3D_LOCATION_BUFFER;
-    }
-
-    volume->container = container;
-
-    return WINED3D_OK;
-}
