@@ -178,11 +178,11 @@ static HIMCC updateCompStr(HIMCC old, LPCWSTR compstr, DWORD len, DWORD *flags)
     {
         needed_size += lpcs->dwCompReadAttrLen;
         needed_size += lpcs->dwCompReadClauseLen;
-        needed_size += lpcs->dwCompReadStrLen * sizeof(DWORD);
+        needed_size += lpcs->dwCompReadStrLen * sizeof(WCHAR);
         needed_size += lpcs->dwResultReadClauseLen;
-        needed_size += lpcs->dwResultReadStrLen * sizeof(DWORD);
+        needed_size += lpcs->dwResultReadStrLen * sizeof(WCHAR);
         needed_size += lpcs->dwResultClauseLen;
-        needed_size += lpcs->dwResultStrLen * sizeof(DWORD);
+        needed_size += lpcs->dwResultStrLen * sizeof(WCHAR);
         needed_size += lpcs->dwPrivateSize;
     }
     rc = ImmCreateIMCC(needed_size);
@@ -271,6 +271,8 @@ static HIMCC updateCompStr(HIMCC old, LPCWSTR compstr, DWORD len, DWORD *flags)
         *(DWORD*)&newdata[current_offset] = len;
         current_offset += sizeof(DWORD);
     }
+    else
+        new_one->dwCompClauseLen = 0;
 
     /* CompStr */
     new_one->dwCompStrLen = len;
@@ -324,12 +326,12 @@ static HIMCC updateResultStr(HIMCC old, LPWSTR resultstr, DWORD len)
     {
         needed_size += lpcs->dwCompReadAttrLen;
         needed_size += lpcs->dwCompReadClauseLen;
-        needed_size += lpcs->dwCompReadStrLen * sizeof(DWORD);
+        needed_size += lpcs->dwCompReadStrLen * sizeof(WCHAR);
         needed_size += lpcs->dwCompAttrLen;
         needed_size += lpcs->dwCompClauseLen;
-        needed_size += lpcs->dwCompStrLen * sizeof(DWORD);
+        needed_size += lpcs->dwCompStrLen * sizeof(WCHAR);
         needed_size += lpcs->dwResultReadClauseLen;
-        needed_size += lpcs->dwResultReadStrLen * sizeof(DWORD);
+        needed_size += lpcs->dwResultReadStrLen * sizeof(WCHAR);
         needed_size += lpcs->dwPrivateSize;
     }
     rc = ImmCreateIMCC(needed_size);
@@ -411,6 +413,8 @@ static HIMCC updateResultStr(HIMCC old, LPWSTR resultstr, DWORD len)
         *(DWORD*)&newdata[current_offset] = len;
         current_offset += sizeof(DWORD);
     }
+    else
+        new_one->dwResultClauseLen = 0;
 
     /* ResultStr */
     new_one->dwResultStrLen = len;
