@@ -31,15 +31,116 @@
 #include "initguid.h"
 DEFINE_GUID(IID_IXmlWriterOutput, 0xc1131708, 0x0f59, 0x477f, 0x93, 0x59, 0x7d, 0x33, 0x24, 0x51, 0xbc, 0x1a);
 
-static HRESULT (WINAPI *pCreateXmlWriter)(REFIID riid, void **ppvObject, IMalloc *pMalloc);
-static HRESULT (WINAPI *pCreateXmlWriterOutputWithEncodingName)(IUnknown *stream,
-                                                                IMalloc *imalloc,
-                                                                LPCWSTR encoding_name,
-                                                                IXmlWriterOutput **output);
-static HRESULT (WINAPI *pCreateXmlWriterOutputWithEncodingCodePage)(IUnknown *stream,
-                                                                    IMalloc *imalloc,
-                                                                    UINT codepage,
-                                                                    IXmlWriterOutput **output);
+static void check_output(IStream *stream, const char *expected, int line)
+{
+    HGLOBAL hglobal;
+    int len = strlen(expected), size;
+    char *ptr;
+    HRESULT hr;
+
+    hr = GetHGlobalFromStream(stream, &hglobal);
+    ok_(__FILE__, line)(hr == S_OK, "got 0x%08x\n", hr);
+
+    size = GlobalSize(hglobal);
+    ptr = GlobalLock(hglobal);
+    if (size != len)
+    {
+        ok_(__FILE__, line)(0, "data size mismatch, expected %u, got %u\n", len, size);
+        ok_(__FILE__, line)(0, "got %s, expected %s\n", ptr, expected);
+    }
+    else
+        ok_(__FILE__, line)(!strncmp(ptr, expected, len), "got %s, expected %s\n", ptr, expected);
+    GlobalUnlock(hglobal);
+}
+#define CHECK_OUTPUT(stream, expected) check_output(stream, expected, __LINE__)
+
+/* used to test all Write* methods for consistent error state */
+static void check_writer_state(IXmlWriter *writer, HRESULT exp_hr)
+{
+    static const WCHAR aW[] = {'a',0};
+    HRESULT hr;
+
+    /* FIXME: add WriteAttributes */
+
+    hr = IXmlWriter_WriteAttributeString(writer, NULL, aW, NULL, aW);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    hr = IXmlWriter_WriteCData(writer, aW);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    hr = IXmlWriter_WriteCharEntity(writer, aW[0]);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    hr = IXmlWriter_WriteChars(writer, aW, 1);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    hr = IXmlWriter_WriteComment(writer, aW);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    /* FIXME: add WriteDocType */
+
+    hr = IXmlWriter_WriteElementString(writer, NULL, aW, NULL, aW);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    hr = IXmlWriter_WriteEndDocument(writer);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    hr = IXmlWriter_WriteEndElement(writer);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    hr = IXmlWriter_WriteEntityRef(writer, aW);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    hr = IXmlWriter_WriteFullEndElement(writer);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    hr = IXmlWriter_WriteName(writer, aW);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    hr = IXmlWriter_WriteNmToken(writer, aW);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    /* FIXME: add WriteNode */
+    /* FIXME: add WriteNodeShallow */
+
+    hr = IXmlWriter_WriteProcessingInstruction(writer, aW, aW);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    hr = IXmlWriter_WriteQualifiedName(writer, aW, NULL);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    hr = IXmlWriter_WriteRaw(writer, aW);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    hr = IXmlWriter_WriteRawChars(writer, aW, 1);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    hr = IXmlWriter_WriteStartDocument(writer, XmlStandalone_Omit);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    hr = IXmlWriter_WriteStartElement(writer, NULL, aW, NULL);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    hr = IXmlWriter_WriteString(writer, aW);
+    ok(hr == exp_hr, "got 0x%08x, expected 0x%08x\n", hr, exp_hr);
+
+    /* FIXME: add WriteSurrogateCharEntity */
+    /* FIXME: add WriteWhitespace */
+}
+
+static IStream *writer_set_output(IXmlWriter *writer)
+{
+    IStream *stream;
+    HRESULT hr;
+
+    hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IXmlWriter_SetOutput(writer, (IUnknown*)stream);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    return stream;
+}
 
 static HRESULT WINAPI testoutput_QueryInterface(IUnknown *iface, REFIID riid, void **obj)
 {
@@ -127,11 +228,11 @@ static void test_writer_create(void)
     /* crashes native */
     if (0)
     {
-        pCreateXmlWriter(&IID_IXmlWriter, NULL, NULL);
-        pCreateXmlWriter(NULL, (void**)&writer, NULL);
+        CreateXmlWriter(&IID_IXmlWriter, NULL, NULL);
+        CreateXmlWriter(NULL, (void**)&writer, NULL);
     }
 
-    hr = pCreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
+    hr = CreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     /* check default properties values */
@@ -158,26 +259,6 @@ static void test_writer_create(void)
     IXmlWriter_Release(writer);
 }
 
-static BOOL init_pointers(void)
-{
-    /* don't free module here, it's to be unloaded on exit */
-    HMODULE mod = LoadLibraryA("xmllite.dll");
-
-    if (!mod)
-    {
-        win_skip("xmllite library not available\n");
-        return FALSE;
-    }
-
-#define MAKEFUNC(f) if (!(p##f = (void*)GetProcAddress(mod, #f))) return FALSE;
-    MAKEFUNC(CreateXmlWriter);
-    MAKEFUNC(CreateXmlWriterOutputWithEncodingName);
-    MAKEFUNC(CreateXmlWriterOutputWithEncodingCodePage);
-#undef MAKEFUNC
-
-    return TRUE;
-}
-
 static void test_writeroutput(void)
 {
     static const WCHAR utf16W[] = {'u','t','f','-','1','6',0};
@@ -186,11 +267,11 @@ static void test_writeroutput(void)
     HRESULT hr;
 
     output = NULL;
-    hr = pCreateXmlWriterOutputWithEncodingName(&testoutput, NULL, NULL, &output);
+    hr = CreateXmlWriterOutputWithEncodingName(&testoutput, NULL, NULL, &output);
     ok(hr == S_OK, "got %08x\n", hr);
     IUnknown_Release(output);
 
-    hr = pCreateXmlWriterOutputWithEncodingName(&testoutput, NULL, utf16W, &output);
+    hr = CreateXmlWriterOutputWithEncodingName(&testoutput, NULL, utf16W, &output);
     ok(hr == S_OK, "got %08x\n", hr);
     unk = NULL;
     hr = IUnknown_QueryInterface(output, &IID_IXmlWriterOutput, (void**)&unk);
@@ -200,11 +281,11 @@ static void test_writeroutput(void)
     IUnknown_Release(output);
 
     output = NULL;
-    hr = pCreateXmlWriterOutputWithEncodingCodePage(&testoutput, NULL, ~0u, &output);
+    hr = CreateXmlWriterOutputWithEncodingCodePage(&testoutput, NULL, ~0u, &output);
     ok(hr == S_OK, "got %08x\n", hr);
     IUnknown_Release(output);
 
-    hr = pCreateXmlWriterOutputWithEncodingCodePage(&testoutput, NULL, CP_UTF8, &output);
+    hr = CreateXmlWriterOutputWithEncodingCodePage(&testoutput, NULL, CP_UTF8, &output);
     ok(hr == S_OK, "got %08x\n", hr);
     unk = NULL;
     hr = IUnknown_QueryInterface(output, &IID_IXmlWriterOutput, (void**)&unk);
@@ -221,12 +302,10 @@ static void test_writestartdocument(void)
     static const WCHAR versionW[] = {'v','e','r','s','i','o','n','=','"','1','.','0','"',0};
     static const WCHAR xmlW[] = {'x','m','l',0};
     IXmlWriter *writer;
-    HGLOBAL hglobal;
     IStream *stream;
     HRESULT hr;
-    char *ptr;
 
-    hr = pCreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
+    hr = CreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     /* output not set */
@@ -239,11 +318,7 @@ static void test_writestartdocument(void)
     hr = IXmlWriter_Flush(writer);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    hr = IXmlWriter_SetOutput(writer, (IUnknown*)stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
+    stream = writer_set_output(writer);
 
     /* nothing written yet */
     hr = IXmlWriter_Flush(writer);
@@ -255,12 +330,7 @@ static void test_writestartdocument(void)
     hr = IXmlWriter_Flush(writer);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    hr = GetHGlobalFromStream(stream, &hglobal);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    ptr = GlobalLock(hglobal);
-    ok(!strncmp(ptr, fullprolog, strlen(fullprolog)), "got %s, expected %s\n", ptr, fullprolog);
-    GlobalUnlock(hglobal);
+    CHECK_OUTPUT(stream, fullprolog);
 
     /* one more time */
     hr = IXmlWriter_WriteStartDocument(writer, XmlStandalone_Yes);
@@ -268,11 +338,7 @@ static void test_writestartdocument(void)
     IStream_Release(stream);
 
     /* now add PI manually, and try to start a document */
-    hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    hr = IXmlWriter_SetOutput(writer, (IUnknown*)stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
+    stream = writer_set_output(writer);
 
     hr = IXmlWriter_WriteProcessingInstruction(writer, xmlW, versionW);
     ok(hr == S_OK, "got 0x%08x\n", hr);
@@ -290,12 +356,7 @@ static void test_writestartdocument(void)
     hr = IXmlWriter_Flush(writer);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    hr = GetHGlobalFromStream(stream, &hglobal);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    ptr = GlobalLock(hglobal);
-    ok(!strncmp(ptr, prologversion, strlen(prologversion)), "got %s\n", ptr);
-    GlobalUnlock(hglobal);
+    CHECK_OUTPUT(stream, prologversion);
 
     IStream_Release(stream);
     IXmlWriter_Release(writer);
@@ -306,7 +367,7 @@ static void test_flush(void)
     IXmlWriter *writer;
     HRESULT hr;
 
-    hr = pCreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
+    hr = CreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     hr = IXmlWriter_SetOutput(writer, (IUnknown*)&teststream);
@@ -342,14 +403,10 @@ static void test_omitxmldeclaration(void)
     HRESULT hr;
     char *ptr;
 
-    hr = pCreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
+    hr = CreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
-    hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    hr = IXmlWriter_SetOutput(writer, (IUnknown*)stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
+    stream = writer_set_output(writer);
 
     hr = IXmlWriter_SetProperty(writer, XmlWriterProperty_OmitXmlDeclaration, TRUE);
     ok(hr == S_OK, "got 0x%08x\n", hr);
@@ -374,11 +431,7 @@ static void test_omitxmldeclaration(void)
     IStream_Release(stream);
 
     /* now add PI manually, and try to start a document */
-    hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    hr = IXmlWriter_SetOutput(writer, (IUnknown*)stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
+    stream = writer_set_output(writer);
 
     hr = IXmlWriter_WriteProcessingInstruction(writer, xmlW, versionW);
     ok(hr == S_OK, "got 0x%08x\n", hr);
@@ -386,12 +439,7 @@ static void test_omitxmldeclaration(void)
     hr = IXmlWriter_Flush(writer);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    hr = GetHGlobalFromStream(stream, &hglobal);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    ptr = GlobalLock(hglobal);
-    ok(!strncmp(ptr, prologversion, strlen(prologversion)), "got %s\n", ptr);
-    GlobalUnlock(hglobal);
+    CHECK_OUTPUT(stream, prologversion);
 
     hr = IXmlWriter_WriteStartDocument(writer, XmlStandalone_Yes);
     ok(hr == S_OK, "got 0x%08x\n", hr);
@@ -399,9 +447,7 @@ static void test_omitxmldeclaration(void)
     hr = IXmlWriter_Flush(writer);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    ptr = GlobalLock(hglobal);
-    ok(!strncmp(ptr, prologversion, strlen(prologversion)), "got %s\n", ptr);
-    GlobalUnlock(hglobal);
+    CHECK_OUTPUT(stream, prologversion);
 
     hr = IXmlWriter_WriteStartDocument(writer, XmlStandalone_Yes);
     ok(hr == WR_E_INVALIDACTION, "got 0x%08x\n", hr);
@@ -409,9 +455,7 @@ static void test_omitxmldeclaration(void)
     hr = IXmlWriter_Flush(writer);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    ptr = GlobalLock(hglobal);
-    ok(!strncmp(ptr, prologversion, strlen(prologversion)), "got %s\n", ptr);
-    GlobalUnlock(hglobal);
+    CHECK_OUTPUT(stream, prologversion);
 
     /* another attempt to add 'xml' PI */
     hr = IXmlWriter_WriteProcessingInstruction(writer, xmlW, versionW);
@@ -440,10 +484,10 @@ static void test_bom(void)
     hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    hr = pCreateXmlWriterOutputWithEncodingName((IUnknown*)stream, NULL, utf16W, &output);
+    hr = CreateXmlWriterOutputWithEncodingName((IUnknown*)stream, NULL, utf16W, &output);
     ok(hr == S_OK, "got %08x\n", hr);
 
-    hr = pCreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
+    hr = CreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     hr = IXmlWriter_SetProperty(writer, XmlWriterProperty_OmitXmlDeclaration, TRUE);
@@ -473,7 +517,7 @@ static void test_bom(void)
     hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    hr = pCreateXmlWriterOutputWithEncodingName((IUnknown*)stream, NULL, utf16W, &output);
+    hr = CreateXmlWriterOutputWithEncodingName((IUnknown*)stream, NULL, utf16W, &output);
     ok(hr == S_OK, "got %08x\n", hr);
 
     hr = IXmlWriter_SetOutput(writer, output);
@@ -499,7 +543,7 @@ static void test_bom(void)
     hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    hr = pCreateXmlWriterOutputWithEncodingName((IUnknown*)stream, NULL, utf16W, &output);
+    hr = CreateXmlWriterOutputWithEncodingName((IUnknown*)stream, NULL, utf16W, &output);
     ok(hr == S_OK, "got %08x\n", hr);
 
     hr = IXmlWriter_SetOutput(writer, output);
@@ -525,7 +569,7 @@ static void test_bom(void)
     hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    hr = pCreateXmlWriterOutputWithEncodingName((IUnknown*)stream, NULL, utf16W, &output);
+    hr = CreateXmlWriterOutputWithEncodingName((IUnknown*)stream, NULL, utf16W, &output);
     ok(hr == S_OK, "got %08x\n", hr);
 
     hr = IXmlWriter_SetOutput(writer, output);
@@ -556,23 +600,17 @@ static void test_writestartelement(void)
     static const char *str = "<a><b>value</b>";
     static const WCHAR aW[] = {'a',0};
     static const WCHAR bW[] = {'b',0};
-    char *ptr;
     IXmlWriter *writer;
     IStream *stream;
-    HGLOBAL hglobal;
     HRESULT hr;
 
-    hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    hr = pCreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
+    hr = CreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     hr = IXmlWriter_WriteStartElement(writer, NULL, aW, NULL);
     ok(hr == E_UNEXPECTED, "got 0x%08x\n", hr);
 
-    hr = IXmlWriter_SetOutput(writer, (IUnknown*)stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
+    stream = writer_set_output(writer);
 
     hr = IXmlWriter_WriteStartElement(writer, aW, NULL, NULL);
     ok(hr == E_INVALIDARG, "got 0x%08x\n", hr);
@@ -586,18 +624,13 @@ static void test_writestartelement(void)
     hr = IXmlWriter_WriteStartElement(writer, NULL, aW, NULL);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    hr = GetHGlobalFromStream(stream, &hglobal);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
     hr = IXmlWriter_WriteStartDocument(writer, XmlStandalone_Yes);
     ok(hr == WR_E_INVALIDACTION, "got 0x%08x\n", hr);
 
     hr = IXmlWriter_Flush(writer);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    ptr = GlobalLock(hglobal);
-    ok(!strncmp(ptr, "<a", 2), "got %s\n", ptr);
-    GlobalUnlock(hglobal);
+    CHECK_OUTPUT(stream, "<a");
 
     hr = IXmlWriter_WriteStartDocument(writer, XmlStandalone_Yes);
     ok(hr == WR_E_INVALIDACTION, "got 0x%08x\n", hr);
@@ -612,17 +645,13 @@ static void test_writestartelement(void)
     IXmlWriter_Release(writer);
 
     /* WriteElementString */
-    hr = pCreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
+    hr = CreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
-    hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
     hr = IXmlWriter_WriteElementString(writer, NULL, bW, NULL, valueW);
     ok(hr == E_UNEXPECTED, "got 0x%08x\n", hr);
 
-    hr = IXmlWriter_SetOutput(writer, (IUnknown*)stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
+    stream = writer_set_output(writer);
 
     hr = IXmlWriter_WriteStartElement(writer, NULL, aW, NULL);
     ok(hr == S_OK, "got 0x%08x\n", hr);
@@ -633,12 +662,7 @@ static void test_writestartelement(void)
     hr = IXmlWriter_Flush(writer);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    hr = GetHGlobalFromStream(stream, &hglobal);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    ptr = GlobalLock(hglobal);
-    ok(!strncmp(ptr, str, strlen(str)), "got %s\n", ptr);
-    GlobalUnlock(hglobal);
+    CHECK_OUTPUT(stream, str);
 
     IStream_Release(stream);
     IXmlWriter_Release(writer);
@@ -648,20 +672,14 @@ static void test_writeendelement(void)
 {
     static const WCHAR aW[] = {'a',0};
     static const WCHAR bW[] = {'b',0};
-    char *ptr;
     IXmlWriter *writer;
     IStream *stream;
-    HGLOBAL hglobal;
     HRESULT hr;
 
-    hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    hr = pCreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
+    hr = CreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
-    hr = IXmlWriter_SetOutput(writer, (IUnknown*)stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
+    stream = writer_set_output(writer);
 
     hr = IXmlWriter_WriteStartElement(writer, NULL, aW, NULL);
     ok(hr == S_OK, "got 0x%08x\n", hr);
@@ -675,15 +693,10 @@ static void test_writeendelement(void)
     hr = IXmlWriter_WriteEndElement(writer);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    hr = GetHGlobalFromStream(stream, &hglobal);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
     hr = IXmlWriter_Flush(writer);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    ptr = GlobalLock(hglobal);
-    ok(!strncmp(ptr, "<a><b /></a>", 12), "got %s\n", ptr);
-    GlobalUnlock(hglobal);
+    CHECK_OUTPUT(stream, "<a><b /></a>");
 
     IXmlWriter_Release(writer);
     IStream_Release(stream);
@@ -699,17 +712,13 @@ static void test_writeenddocument(void)
     HRESULT hr;
     char *ptr;
 
-    hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    hr = pCreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
+    hr = CreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     hr = IXmlWriter_WriteEndDocument(writer);
     ok(hr == E_UNEXPECTED, "got 0x%08x\n", hr);
 
-    hr = IXmlWriter_SetOutput(writer, (IUnknown*)stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
+    stream = writer_set_output(writer);
 
     /* WriteEndDocument resets it to initial state */
     hr = IXmlWriter_WriteEndDocument(writer);
@@ -746,10 +755,7 @@ static void test_writeenddocument(void)
     hr = IXmlWriter_Flush(writer);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    ptr = GlobalLock(hglobal);
-    ok(ptr != NULL, "got %p\n", ptr);
-    ok(!strncmp(ptr, "<a><b /></a>", 12), "got %s\n", ptr);
-    GlobalUnlock(hglobal);
+    CHECK_OUTPUT(stream, "<a><b /></a>");
 
     IXmlWriter_Release(writer);
     IStream_Release(stream);
@@ -762,14 +768,9 @@ static void test_WriteComment(void)
     static const WCHAR bW[] = {'b',0};
     IXmlWriter *writer;
     IStream *stream;
-    HGLOBAL hglobal;
     HRESULT hr;
-    char *ptr;
 
-    hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    hr = pCreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
+    hr = CreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     hr = IXmlWriter_SetProperty(writer, XmlWriterProperty_OmitXmlDeclaration, TRUE);
@@ -778,8 +779,7 @@ static void test_WriteComment(void)
     hr = IXmlWriter_WriteComment(writer, aW);
     ok(hr == E_UNEXPECTED, "got 0x%08x\n", hr);
 
-    hr = IXmlWriter_SetOutput(writer, (IUnknown*)stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
+    stream = writer_set_output(writer);
 
     hr = IXmlWriter_WriteStartDocument(writer, XmlStandalone_Omit);
     ok(hr == S_OK, "got 0x%08x\n", hr);
@@ -802,13 +802,7 @@ static void test_WriteComment(void)
     hr = IXmlWriter_Flush(writer);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    hr = GetHGlobalFromStream(stream, &hglobal);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    ptr = GlobalLock(hglobal);
-    ok(ptr != NULL, "got %p\n", ptr);
-    ok(!strncmp(ptr, "<!--a--><b><!--a--><!----><!--- ->-->", 37), "got %s\n", ptr);
-    GlobalUnlock(hglobal);
+    CHECK_OUTPUT(stream, "<!--a--><b><!--a--><!----><!--- ->-->");
 
     IXmlWriter_Release(writer);
     IStream_Release(stream);
@@ -822,14 +816,9 @@ static void test_WriteCData(void)
     static const WCHAR bW[] = {'b',0};
     IXmlWriter *writer;
     IStream *stream;
-    HGLOBAL hglobal;
     HRESULT hr;
-    char *ptr;
 
-    hr = CreateStreamOnHGlobal(NULL, TRUE, &stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    hr = pCreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
+    hr = CreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
     ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
 
     hr = IXmlWriter_SetProperty(writer, XmlWriterProperty_OmitXmlDeclaration, TRUE);
@@ -838,8 +827,7 @@ static void test_WriteCData(void)
     hr = IXmlWriter_WriteCData(writer, aW);
     ok(hr == E_UNEXPECTED, "got 0x%08x\n", hr);
 
-    hr = IXmlWriter_SetOutput(writer, (IUnknown*)stream);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
+    stream = writer_set_output(writer);
 
     hr = IXmlWriter_WriteStartElement(writer, NULL, bW, NULL);
     ok(hr == S_OK, "got 0x%08x\n", hr);
@@ -859,33 +847,169 @@ static void test_WriteCData(void)
     hr = IXmlWriter_Flush(writer);
     ok(hr == S_OK, "got 0x%08x\n", hr);
 
-    hr = GetHGlobalFromStream(stream, &hglobal);
-    ok(hr == S_OK, "got 0x%08x\n", hr);
-
-    ptr = GlobalLock(hglobal);
-    ok(ptr != NULL, "got %p\n", ptr);
-
-    ok(!strncmp(ptr,
+    CHECK_OUTPUT(stream,
         "<b>"
         "<![CDATA[a]]>"
         "<![CDATA[]]>"
         "<![CDATA[]]]]>"
         "<![CDATA[>]]>"
         "<![CDATA[a]]]]>"
-        "<![CDATA[>b]]>", 84), "got %s\n", ptr);
-
-    GlobalUnlock(hglobal);
+        "<![CDATA[>b]]>");
 
     IXmlWriter_Release(writer);
     IStream_Release(stream);
 }
 
+static void test_WriteRaw(void)
+{
+    static const WCHAR rawW[] = {'a','<',':',0};
+    static const WCHAR aW[] = {'a',0};
+    IXmlWriter *writer;
+    IStream *stream;
+    HRESULT hr;
+
+    hr = CreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
+    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+
+    hr = IXmlWriter_WriteRaw(writer, NULL);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IXmlWriter_WriteRaw(writer, rawW);
+    ok(hr == E_UNEXPECTED, "got 0x%08x\n", hr);
+
+    stream = writer_set_output(writer);
+
+    hr = IXmlWriter_WriteRaw(writer, NULL);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IXmlWriter_WriteRaw(writer, rawW);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IXmlWriter_WriteRaw(writer, rawW);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IXmlWriter_WriteComment(writer, rawW);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IXmlWriter_WriteRaw(writer, rawW);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IXmlWriter_WriteElementString(writer, NULL, aW, NULL, aW);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    hr = IXmlWriter_WriteStartDocument(writer, XmlStandalone_Yes);
+    ok(hr == WR_E_INVALIDACTION, "got 0x%08x\n", hr);
+
+    hr = IXmlWriter_WriteComment(writer, rawW);
+    ok(hr == WR_E_INVALIDACTION, "got 0x%08x\n", hr);
+
+    hr = IXmlWriter_WriteEndDocument(writer);
+    ok(hr == WR_E_INVALIDACTION, "got 0x%08x\n", hr);
+
+    hr = IXmlWriter_WriteRaw(writer, rawW);
+    ok(hr == WR_E_INVALIDACTION, "got 0x%08x\n", hr);
+
+    hr = IXmlWriter_Flush(writer);
+    ok(hr == S_OK, "got 0x%08x\n", hr);
+
+    CHECK_OUTPUT(stream, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>a<:a<:<!--a<:-->a<:<a>a</a>");
+
+    IXmlWriter_Release(writer);
+    IStream_Release(stream);
+}
+
+static void test_writer_state(void)
+{
+    static const WCHAR aW[] = {'a',0};
+    IXmlWriter *writer;
+    IStream *stream;
+    HRESULT hr;
+
+    hr = CreateXmlWriter(&IID_IXmlWriter, (void**)&writer, NULL);
+    ok(hr == S_OK, "Expected S_OK, got %08x\n", hr);
+
+    /* initial state */
+    check_writer_state(writer, E_UNEXPECTED);
+
+    /* set output and call 'wrong' method, WriteEndElement */
+    stream = writer_set_output(writer);
+
+    hr = IXmlWriter_WriteEndElement(writer);
+    ok(hr == WR_E_INVALIDACTION, "got 0x%08x\n", hr);
+
+    check_writer_state(writer, WR_E_INVALIDACTION);
+    IStream_Release(stream);
+
+    /* WriteAttributeString */
+    stream = writer_set_output(writer);
+
+    hr = IXmlWriter_WriteAttributeString(writer, NULL, aW, NULL, aW);
+    ok(hr == WR_E_INVALIDACTION, "got 0x%08x\n", hr);
+
+    check_writer_state(writer, WR_E_INVALIDACTION);
+    IStream_Release(stream);
+
+    /* WriteEndDocument */
+    stream = writer_set_output(writer);
+
+    hr = IXmlWriter_WriteEndDocument(writer);
+    ok(hr == WR_E_INVALIDACTION, "got 0x%08x\n", hr);
+
+    check_writer_state(writer, WR_E_INVALIDACTION);
+    IStream_Release(stream);
+
+    /* WriteFullEndElement */
+    stream = writer_set_output(writer);
+
+    hr = IXmlWriter_WriteFullEndElement(writer);
+    ok(hr == WR_E_INVALIDACTION, "got 0x%08x\n", hr);
+
+    check_writer_state(writer, WR_E_INVALIDACTION);
+    IStream_Release(stream);
+
+    /* WriteCData */
+    stream = writer_set_output(writer);
+
+    hr = IXmlWriter_WriteCData(writer, aW);
+    ok(hr == WR_E_INVALIDACTION, "got 0x%08x\n", hr);
+
+    check_writer_state(writer, WR_E_INVALIDACTION);
+    IStream_Release(stream);
+
+    /* WriteName */
+    stream = writer_set_output(writer);
+
+    hr = IXmlWriter_WriteName(writer, aW);
+    ok(hr == WR_E_INVALIDACTION, "got 0x%08x\n", hr);
+
+    check_writer_state(writer, WR_E_INVALIDACTION);
+    IStream_Release(stream);
+
+    /* WriteNmToken */
+    stream = writer_set_output(writer);
+
+    hr = IXmlWriter_WriteNmToken(writer, aW);
+    ok(hr == WR_E_INVALIDACTION, "got 0x%08x\n", hr);
+
+    check_writer_state(writer, WR_E_INVALIDACTION);
+    IStream_Release(stream);
+
+    /* WriteString */
+    stream = writer_set_output(writer);
+
+    hr = IXmlWriter_WriteString(writer, aW);
+    ok(hr == WR_E_INVALIDACTION, "got 0x%08x\n", hr);
+
+    check_writer_state(writer, WR_E_INVALIDACTION);
+    IStream_Release(stream);
+
+    IXmlWriter_Release(writer);
+}
+
 START_TEST(writer)
 {
-    if (!init_pointers())
-       return;
-
     test_writer_create();
+    test_writer_state();
     test_writeroutput();
     test_writestartdocument();
     test_writestartelement();
@@ -896,4 +1020,5 @@ START_TEST(writer)
     test_writeenddocument();
     test_WriteComment();
     test_WriteCData();
+    test_WriteRaw();
 }
